<?php
/**
 * The theme admin functions.
 *
 * @package Vantage
 *
 * @since 1.0.0
 */

/**
 * Enqueue scripts for admin pages.
 *
 * @since 1.x.x
 *
 * @param string $hook_suffix The current admin page.
 */
function va_add_admin_scripts( $hook_suffix ) {
	global $post;

	wp_enqueue_style( 'font-awesome' );

	if ( empty( $post ) || VA_LISTING_PTYPE != $post->post_type ) {
		return;
	}

	// Selectively load on certain pages.
	$pages = array( 'edit.php', 'post.php', 'post-new.php', 'media-upload-popup' );

	if ( ! in_array( $hook_suffix, $pages ) ) {
		return;
	}

	// Load the supporting js script so we can perform magic in wp-admin.
	wp_enqueue_script( 'va-admin-listing-edit', get_template_directory_uri() . '/includes/admin/scripts/listing-edit.js', array(), VA_VERSION, true );
}
add_action( 'admin_enqueue_scripts', 'va_add_admin_scripts', 10 );

/**
 * Limit wp-admin access to only admins with 'manage_options' capability.
 *
 * @since 1.0.0
 * @since 4.0.0 Added filter and set default capability
 */
function va_restrict_admin_access() {
	/**
	 * Filter the allowed capability to access wp-admin.
	 *
	 * @url https://codex.wordpress.org/Roles_and_Capabilities
	 *
	 * @since 4.0.0
	 *
	 * @param string $capability The minimum WordPress capability allowed.
	 *                           Defaults to 'manage_options'.
	 */
	$capability = apply_filters( 'va_restrict_admin_access', 'manage_options' );

	if ( ! ( defined( 'DOING_AJAX' ) && DOING_AJAX ) && ! current_user_can( $capability ) ) {
		wp_redirect( site_url() );
		exit;
	}
}
add_action( 'admin_init', 'va_restrict_admin_access' );

/**
 * Excludes Vantage theme from checking for updates in WP API
 *
 * See: https://core.trac.wordpress.org/ticket/14179
 *
 * @since 1.x.x
 */
function _va_exclude_theme_from_wp_api( $r, $url ) {

	// AppThemes Updater present, no need to exclude.
	if ( function_exists( 'app_updater_activate' ) ) {
		return $r;
	}

	if ( preg_match( '#://api\.wordpress\.org/themes/update-check/(?P<version>[0-9.]+)/#', $url, $matches ) ) {

		if ( $matches['version'] < 1.1 ) {
			return $r;
		}

		$themes = json_decode( $r['body']['themes'], true );
		if ( empty( $themes ) ) {
			return $r;
		}

		if ( isset( $themes['themes']['vantage'] ) ) {
			unset( $themes['themes']['vantage'] );
		}

		$r['body']['themes'] = json_encode( $themes );
	}

	return $r;
}
add_filter( 'http_request_args', '_va_exclude_theme_from_wp_api', 10, 2 );

/**
 * Saves permalink settings.
 *
 * Temporary workaround for wordpress bug #9296
 * Although there is a hook in the options-permalink.php to insert custom settings,
 * it does not actually save any custom setting which is added to that page.
 *
 * @see http://core.trac.wordpress.org/ticket/9296
 *
 * @since 1.0.0
 *
 * @return void
 */
function va_enable_permalink_settings() {
	global $new_whitelist_options;

	// Save hook for permalinks page.
	if ( isset( $_POST['permalink_structure'] ) || isset( $_POST['category_base'] ) ) {
		check_admin_referer( 'update-permalink' );

		$option_page = 'permalink';

		$capability = 'manage_options';
		$capability = apply_filters( "option_page_capability_{$option_page}", $capability );

		if ( ! current_user_can( $capability ) ) {
			wp_die( __( 'Cheatin&#8217; uh?', APP_TD ) );
		}

		// Get extra permalink options.
		$options = $new_whitelist_options[ $option_page ];

		if ( $options ) {
			foreach ( $options as $option ) {
				$option = trim( $option );
				$value  = null;

				if ( isset( $_POST[ $option ] ) ) {
					$value = $_POST[ $option ];
				}

				if ( ! is_array( $value ) ) {
					$value = trim( $value );
				}

				$value = stripslashes_deep( $value );

				// Get the old values to merge.
				$db_option = get_option( $option );

				if ( is_array( $db_option ) ) {
					update_option( $option, array_merge( $db_option, $value ) );
				} else {
					update_option( $option, $value );
				}

				// Flush rewrite rules using a transient
				//set_transient( 'va_flush_rewrite_rules', 1, 300 );
			}

			// Yes, we need to do this now, come back to this and make the transient work in the right timing.
			flush_rewrite_rules();
		}

		/**
		 *  Handle settings errors
		 */
		set_transient( 'settings_errors', get_settings_errors(), 30 );
	}
}
// Add a very low priority action to make sure any extra settings have been added to the permalinks global.
add_action( 'admin_init', 'va_enable_permalink_settings', 999999 );

function va_upgrade_user_default_role() {
	global $va_options;

	list( $args ) = get_theme_support( 'app-versions' );
	$previous_version = get_option( $args['option_key'] );

	if ( ! $previous_version ) {
		$va_options->default_user_role_update_1_3_2 = true;
	}
}
add_action( 'appthemes_first_run', 'va_upgrade_user_default_role' );


function va_display_default_role_notice() {
	global $va_options, $pagenow;

	if ( $va_options->default_user_role_update_1_3_2 || isset( $_GET['firstrun'] ) || ! current_user_can( 'manage_options' ) ) {
		return;
	}

	if ( ( ! isset( $_GET['page'] ) || $_GET['page'] != 'app-settings' ) && $pagenow != 'users.php' ) {
		return;
	}

	$ignore_url = 'admin.php?page=app-settings&va_user_roles_ignore=1';
	echo scb_admin_notice( sprintf( __( 'Default User role was changed from "Subscriber" to "Contributor". Please use bulk action "Change role to..." on <a href="%s">Users</a> page to grant existing users "Contributor" role.', APP_TD ), admin_url( 'users.php' ) ) . ' | <a href="' . admin_url( $ignore_url ) . '">' . __( 'Hide Notice', APP_TD ) . '</a>' );
}
add_action( 'admin_notices', 'va_display_default_role_notice', 99 );
