<?php
/**
 * Admin settings page setup.
 *
 * @package Framework\Settings
 */

/**
 * Setup the admin pages class.
 */
class VA_Admin_Settings extends APP_Tabs_Page {

	protected $permalink_sections;
	protected $permalink_options;

	/**
	 * Constructor
	 */
	public function setup() {

		$this->textdomain = APP_TD;

		$this->args = array(
			'page_title'            => __( 'Vantage Settings', APP_TD ),
			'menu_title'            => __( 'Settings', APP_TD ),
			'page_slug'             => 'app-settings',
			'parent'                => 'app-dashboard',
			'screen_icon'           => 'options-general',
			'admin_action_priority' => 10,
		);

		add_action( 'admin_notices', array( $this, 'admin_tools' ) );
		add_action( 'admin_init',    array( $this, 'init_integrated_options' ) );
	}

	public function admin_tools() {
		global $va_options;

		if ( isset( $_GET['prune'] ) && $_GET['prune'] == 1 && isset( $_GET['tab'] ) && $_GET['tab'] == 'advanced' ) {
			va_prune_expired_listings();
			appthemes_prune_addons();
			echo scb_admin_notice( __( 'Expired listings have been pruned', APP_TD ) );
		}

		if ( isset( $_GET['va_user_roles_ignore'] ) ) {
			$va_options->default_user_role_update_1_3_2 = true;
		}
	}

	protected function init_tabs() {

		// Remove unwanted query args from urls. Such a hack! Please change.
		$_SERVER[ 'REQUEST_URI' ] = remove_query_arg( array( 'enabled', 'prune', 'va_user_roles_ignore', 'firstrun' ), $_SERVER[ 'REQUEST_URI' ] );

		$this->tabs->add( 'general',  __( 'General', APP_TD ) );
		$this->tabs->add( 'advanced', __( 'Advanced', APP_TD ) );

		$this->tab_sections['general']['appearance'] = array(
			'title'  => __( 'Appearance', APP_TD ),
			'fields' => array(
				array(
					'title' => __( 'Listings Per Page', APP_TD ),
					'type'  => 'select',
					'name'  => 'listings_per_page',
					'values' => array(
						'5'  => 5,
						'10' => 10,
						'15' => 15,
						'20' => 20,
						'25' => 25,
						'50' => 50,
					),
					'desc'  => __( 'How many listings per page to show', APP_TD ),
					'tip'   => '',
				),
			),
		);

		$this->tab_sections['advanced']['maintenance'] = array(
			'title'   => __( 'Maintenance', APP_TD ),
			'fields'  => array(
				array(
					'title' => __( 'Prune Listings', APP_TD ),
					'name'  => '_blank',
					'type'  => '',
					'desc'  => sprintf( __( 'Prune  <a href="%s">expired listings</a>', APP_TD ), 'admin.php?page=app-settings&tab=advanced&prune=1' ),
					'extra' => array(
						'style' => 'display: none;',
					),
					'tip' => __( 'Manually prune any expired listings. This will run only one time.', APP_TD ),
				),
			),
		);

		$this->tab_sections['advanced']['user'] = array(
			'title'  => __( 'User', APP_TD ),
			'fields' => array(
			array(
					'title' => __( 'Login Page', APP_TD ),
					'type'  => 'checkbox',
					'name'  => 'wp_login',
					'desc'  => __( 'Use the default WordPress login and registration pages', APP_TD ),
					'tip'   => __( "Left unchecked, you'll use the theme's custom styled pages.", APP_TD ),
				),
			),
		);

	}

	function init_integrated_options() {

		// display additional section on the permalinks page
		$this->permalink_sections();

	}

	/**
	 * Setup the custom permalinks section.
	 *
	 * @since 1.1.0
	 *
	 */
	function permalink_sections() {

		$permalink_sections = array(
			'listings'  => __( 'Vantage Permalinks', APP_TD ),
		);

		/**
		 * Filter the listing permalink section titles.
		 *
		 * @since 4.0.0
		 *
		 * @param array $permalink_sections An array of permalink section titles.
		 *
		 */
		$this->permalink_sections = apply_filters( 'va_permalink_sections', $permalink_sections );


		$permalink_options = array(
			'listing_permalink'            => __( 'Listing base', APP_TD ),
			'listing_cat_permalink'        => __( 'Listing category base', APP_TD ),
			'listing_tag_permalink'        => __( 'Listing tag base', APP_TD ),
			'dashboard_permalink'          => __( 'User base', APP_TD ),
			'dashboard_listings_permalink' => __( 'User listing base', APP_TD ),
			'dashboard_claimed_permalink'  => __( 'User claimed base', APP_TD ),
			'dashboard_faves_permalink'    => __( 'User favorites base', APP_TD ),
		);

		/**
		 * Filter the listing permalink options.
		 *
		 * @since 4.0.0
		 *
		 * @param array $permalink_options An array of permalink values.
		 *
		 */
		$this->permalink_options['listings'] = apply_filters( 'va_permalink_options', $permalink_options );

		// Register a setting and its sanitization callback.
		register_setting( 'permalink', 'va_options', array( $this, 'permalink_options_validate' ) );

		foreach ( $this->permalink_sections as $section => $title ) {

			// Add a new section to a settings page.
			add_settings_section( $section, $title, '__return_false', 'permalink' );

			// Add new settings fields.
			foreach ( $this->permalink_options[$section] as $id => $title ) {
				add_settings_field( "va_options_$id", $title, array( $this, 'permalink_section_add_option' ), 'permalink', $section, array( 'id' => $id ) );
			}
		}
	}

	function permalink_section_add_option( $option ) {
		global $va_options;

		echo scbForms::input( array(
			'type'  => 'text',
			'name'  => 'va_options['.$option['id'].']',
			'extra' => array(
				'class' => 'regular-text code',
			),
			'value' => $va_options->{$option['id']},
		) );

	}

	// validate/sanitize permalinks
	function permalink_options_validate( $input ) {
		global $va_options;

		$error_html_id = '';

		foreach ( $this->permalink_sections as $section => $title ) {

			foreach ( $this->permalink_options[ $section ] as $key => $value ) {

				$r = trim( $input[ $key ] );

				if ( empty( $r ) ) {
					$error_html_id = $key;
					// Set option to previous value.
					$input[ $key ] = $va_options->$key;
				} else {
					if ( ! is_array( $input[ $key ] ) ) {
						$input[ $key ] = trim( $input[ $key ] );
					}
					$input[ $key ] = stripslashes_deep( $input[ $key ] );
				}
			}
		}

		if ( $error_html_id ) {

			add_settings_error(
				'va_options',
				$error_html_id,
				__( 'Vantage permalinks cannot be empty. Empty fields have been reverted back to their previous value.', APP_TD ),
				'error'
			);

		}

		return $input;
	}
}
