<?php
/**
 * Main theme categories file.
 *
 * @package Vantage
 * @since 1.0.0
 */

/**
 * Create the categories tree list.
 *
 * @since 1.0.0
 * @since 4.0.0 Refactored. Removed admin options. Replaced with filters.
 *
 * @param string $location The location this structure is being used.
 *
 * @return string
 */
function va_cat_menu_drop_down( $location = 'menu' ) {

	$args = array(
		'hide_empty'     => false,
		'menu_cols'      => ( $location == 'menu' ? 3 : 2 ), // AppThemes custom.
		'show_children'  => true, // AppThemes custom.
		'show_count'     => true, // AppThemes custom.
		'app_pad_counts' => true, // AppThemes custom. Do we still need as of 4.0?
	);

	/**
	 * Filters the array of listing category term arguments.
	 *
	 * The dynamic portion of the filter name, `$location`
	 * refers to the location it's being used.
	 *
	 * @since 4.0.0
	 *
	 * @param array $args An array of arguments. @see get_terms()
	 */
	$args = apply_filters( "va_listing_categories_{$location}_args", $args );

	$terms = get_terms( VA_LISTING_CATEGORY, $args );

	// Get all terms for the taxonomy.
	if ( empty( $terms ) ) {
		return;
	}

	$cats       = array();
	$child_cats = array();

	// Separate into cats and subcats arrays.
	foreach ( $terms as $key => $value ) {

		if ( $value->parent == 0 ) {
			$cats[ $key ] = $terms[ $key ];
		} else {
			$child_cats[ $key ] = $terms[ $key ];
		}
		unset( $terms[ $key ] );
	}

	$i               = 0;
	$cat_menu        = '';
	$total_main_cats = count( $cats ); // total number of parent cats
	$cats_per_col    = ceil( $total_main_cats / $args['menu_cols'] ); // parent cats per column
	$column_cells    = ceil( 12 / $args['menu_cols'] ); // foundation framework div column width

	// Loop through all the cats.
	foreach ( $cats as $cat ) :

		if ( ( $i == 0 ) || ( $i == $cats_per_col ) || ( $i == ( $cats_per_col * 2 ) ) || ( $i == ( $cats_per_col * 3 ) ) ) {
			$cat_menu .= '<div class="cat-column medium-'. esc_attr( $column_cells ) .' columns">';
		}

		// Only show the total count if option is set.
		$show_count = $args['show_count'] ? ' <span class="cat-item-count label">' . $cat->count . '</span>' : '';

		$cat_menu .= '<div class="parent-cat-wrap">';
		$cat_menu .= '<div class="parent-cat cat-item-' . esc_attr( $cat->term_id ) . '"><a href="' . esc_url( get_term_link( $cat, VA_LISTING_CATEGORY ) ) . '" title="' . esc_attr( $cat->description ) . '">' . $cat->name . $show_count . '</a></div><!-- .parent-cat -->';

		// If there are child cats and we want to see them, add them.
		if ( true === $args['show_children'] ) {
			if ( $child_menu = va_create_child_list( $child_cats, $cat->term_id, $location ) ) {
				$cat_menu .= $child_menu;
			}
		}

		$cat_menu .= '</div><!-- .parent-cat-wrap -->';

		if ( ( $i == ( $cats_per_col - 1 ) ) || ( $i == ( ( $cats_per_col * 2 ) - 1 ) ) || ( $i == ( ( $cats_per_col * 3 ) - 1 ) ) || ( $i == ( $total_main_cats - 1 ) ) ) {
			$cat_menu .= '</div><!-- .cat-column -->';
		}

		$i++;

	endforeach;

	return $cat_menu;
}

/**
 * Creates child list, helper function for va_cat_menu_drop_down().
 *
 * Replaces the appthemes_create_child_list() function since we needed to clean
 * it up and use divs vs ul because of foundation drop-down menu conflicts.
 *
 * @since 4.0.0
 *
 * @param array  $cats      The list of child categories.
 * @param int    $parent_id The category parent id.
 * @param string $location  The location this structure is being used.
 *
 * @return string|bool
 */
function va_create_child_list( $cats = array(), $parent_id = 0, $location = '' ) {

	$args = array(
		'taxonomy'    => VA_LISTING_CATEGORY,
		'parent'      => $parent_id,
		'child_count' => true,
		'hide_empty'  => false,
		'number'      => 3, // AppThemes custom.
	);

	/**
	 * Filters the array of listing category child term arguments.
	 *
	 * The dynamic portion of the filter name, `$location`
	 * refers to the location it's being used.
	 *
	 * @since 4.0.0
	 *
	 * @param array $child_args An array of child arguments. @see get_terms()
	 */
	$args = apply_filters( "va_listing_categories_{$location}_child_args", $args );

	$i          = 0;
	$child_menu = '';

	foreach ( $cats as $cat ) :

		if ( $cat->parent == $args['parent'] ) {

			// Hide empty sub cats if option is set.
			if ( $args['hide_empty'] && $cat->count == 0 ) {
				continue;
			}

			// Limit number of sub-categories.
			if ( $i >= $args['number'] ) {
				continue;
			}

			$i++;

			// Show the count if option is set.
			$show_count = $args['child_count'] ? '<span class="cat-item-count label">' . $cat->count . '</span>' : '';

			$child_menu .= '<div class="cat-item cat-item-' . esc_attr( $cat->term_id ) . '"><a href="' . esc_url( get_term_link( $cat, $args['taxonomy'] ) ) . '" title="' . esc_attr( $cat->description ) . '">' . $cat->name . $show_count . '</a>';

			// Send through self to create sub-cats.
			if ( $output = va_create_child_list( $cats, $args ) ) {
				$child_menu .= $output;
			}

			$child_menu .= '</div>';
		}

	endforeach;

	if ( ! empty( $child_menu ) ) {
		return '<div class="sub-cat-list">' . $child_menu . '</div>';
	}

	return false;
}

/**
 * Appends the listing categories mega menu to the categories menu item.
 *
 * @since 4.0.0
 *
 * @param string $item_output The menu item's starting HTML output.
 * @param object $item        Menu item data object.
 * @param int    $depth       Depth of menu item. Used for padding.
 * @param array  $args        An array of wp_nav_menu() arguments.
 */
function va_category_mega_menu( $item_output, $item, $depth, $args ) {

	// Must be a page and in the secondary menu location.
	if ( 'page' != $item->object || 'secondary' != $args->theme_location ) {
		return $item_output;
	}

	// Get the post content so we can look for the categories shortcode.
	$content = get_post_field( 'post_content', $item->object_id );

	// If it's the categories menu item, append the mega menu.
	if ( has_shortcode( $content, 'vantage_listing_categories' ) ) {
		$item_output .= sprintf(
			apply_filters( 'va_category_mega_menu', '<ul class="listing-cats listing-cats-dropdown"><div class="row collapse">%s</div></ul>' ),
			va_cat_menu_drop_down( 'menu' )
		);
	}

	return $item_output;
}
add_filter( 'walker_nav_menu_start_el', 'va_category_mega_menu', 10, 4 );
