<?php
/**
 * Structured data markup class.
 *
 * @package Vantage\StructuredData
 * @author AppThemes
 * @since 4.0.0
 */

/**
 * Main class to hook into core structured data class.
 *
 * @since 4.0.0
 * @see APP_Structured_Data
 */
class VA_Structured_Data {

	/**
	 * Constructor
	 */
	public function __construct() {

		$schema = APP_Structured_Data::instance();

		add_action( 'appthemes_schema_build_data', array( $this, 'build_schema' ) );

		// Parent methods for blog posts.
		add_filter( 'appthemes_schema_type_blogposting',   array( $schema, 'add_comments' ) );
		add_filter( 'appthemes_schema_type_blogposting',   array( $schema, 'add_tags' ) );
		add_filter( 'appthemes_schema_type_blogposting',   array( $schema, 'add_categories' ) );
		add_filter( 'appthemes_schema_type_blogposting',   array( $schema, 'add_word_count' ) );

		// Parent methods for pages.
		add_filter( 'appthemes_schema_type_webpage',       array( $schema, 'add_comments' ) );

		// Self methods used in this theme.
		add_filter( 'appthemes_schema_type_localbusiness', array( $this, 'add_address' ) );
		add_filter( 'appthemes_schema_type_localbusiness', array( $this, 'add_geocoordinates' ) );
		add_filter( 'appthemes_schema_type_localbusiness', array( $this, 'add_photos' ) );
		add_filter( 'appthemes_schema_type_localbusiness', array( $this, 'add_phone' ) );
		add_filter( 'appthemes_schema_type_localbusiness', array( $this, 'add_listing_social_networks' ) );
		add_filter( 'appthemes_schema_type_localbusiness', array( $schema, 'add_comments' ) );

		add_filter( 'appthemes_schema_type_person',        array( $this, 'add_author_social_networks' ) );
	}

	/**
	 * Build the json-ld schema.org output based on page being viewed.
	 *
	 * @since 4.0.0
	 */
	public function build_schema() {

		$schema = APP_Structured_Data::instance();

		if ( is_front_page() ) {
			$schema->data[] = APP_Schema_Type_Website::type();
			$schema->data[] = $schema->organization_home();
		}

		if ( is_singular( 'post' ) ) {
			$schema->data[] = APP_Schema_Type_BlogPosting::type();
		}

		if ( is_singular( VA_LISTING_PTYPE ) ) {

			// @todo Pass in a custom localbusiness type via cat tax custom field.
			$type = 'LocalBusiness';

			$schema->data[] = APP_Schema_Type_LocalBusiness::type( $type );
		}

		if ( is_author() ) {
			$schema->data[] = $schema->author();
		}

		if ( is_page() ) {
			$schema->data[] = APP_Schema_Type_WebPage::type();
		}
	}

	/**
	 * Add the address to the listing json-ld schema if we have any.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of LocalBusiness type schema properties.
	 * @return array
	 */
	public function add_address( $output ) {

		if ( ! is_array( $address = APP_Schema_Type_PostalAddress::type() ) ) {
			return $output;
		}

		// Set the array key.
		$r['address'] = $address;

		return array_merge( $output, $r );
	}

	/**
	 * Add the geo coordinates to the listing json-ld schema if we have any.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of LocalBusiness type schema properties.
	 * @return array
	 */
	public function add_geocoordinates( $output ) {

		if ( ! is_array( $geocoordinates = APP_Schema_Type_GeoCoordinates::type() ) ) {
			return $output;
		}

		// Set the array key.
		$r['geo'] = $geocoordinates;

		return array_merge( $output, $r );
	}

	/**
	 * Add the main photo to the listing json-ld schema if we have any.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of LocalBusiness type schema properties.
	 * @return array
	 */
	public function add_photos( $output ) {

		if ( ! is_array( $photos = APP_Schema_Type_ImageObject::type() ) ) {
			return $output;
		}

		// Set the array key.
		$r['image'] = $photos;

		return array_merge( $output, $r );
	}

	/**
	 * Add the phone number to the listing json-ld schema.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of LocalBusiness type schema properties.
	 * @return array
	 */
	public function add_phone( $output ) {
		global $post;

		// Add the phone number.
		if ( ! $phone = get_post_meta( $post->ID, 'phone', true ) ) {
			return $output;
		}

		// Set the array key.
		$r['telephone'] = $phone;

		return array_merge( $output, $r );
	}

	/**
	 * Add the social networks to the listing json-ld schema.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of LocalBusiness type schema properties.
	 * @return array
	 */
	public function add_listing_social_networks( $output ) {
		global $post;

		// Add any social networks that have values.
		if ( ! array_filter( $social_networks = va_get_available_listing_networks( true ) ) ) {
			return $output;
		}

		foreach ( $social_networks as $key => $value ) {
			$s[] = $value;
		}

		// Run esc_url() on all array values.
		$r['sameAs'] = array_map( 'esc_url', $s );

		return array_merge( $output, $r );
	}

	/**
	 * Add the user contact methods to the output filter.
	 *
	 * @since 4.0.0
	 *
	 * @param array $output List of Person type schema properties.
	 * @return array
	 */
	public function add_author_social_networks( $output ) {

		// Add any social networks that have values.
		if ( ! array_filter( $social_networks = va_get_available_user_networks( true ) ) ) {
			return $output;
		}

		foreach ( $social_networks as $key => $value ) {
			$s[] = $value;
		}

		// Run esc_url() on all array values.
		$r['sameAs'] = array_map( 'esc_url', $s );

		return array_merge( $output, $r );
	}

}

new VA_Structured_Data;
