<?php
/**
 * Listing Settings submodule
 *
 * @package Listing\Admin\Settings
 * @author  AppThemes
 * @since   Listing 1.0
 */

/**
 * Listing Settings class
 *
 * Requires theme supports:
 *	- app-framework
 *  - app-dashboard
 */
class APP_Listing_Settings {

	/**
	 * Current Listing module object
	 * @var APP_Listing
	 */
	protected $listing;

	/**
	 * The list of enabled features to compare against updated. If the lists
	 * different so needs to reload page to enable/disable them.
	 *
	 * @var array
	 */
	protected $enabled_features;

	/**
	 * Construct Listing Settings module
	 *
	 * @param APP_Listing $listing Listing module object.
	 */
	public function __construct( APP_Listing $listing ) {

		$this->listing = $listing;

		add_action( 'admin_init', array( $this, 'init' ) );
	}

	/**
	 * Initiates Listing Settings module
	 *
	 * @global array $admin_page_hooks Registered page hooks.
	 */
	public function init() {
		global $admin_page_hooks;

		$priority = ( 'APP_Listing_Settings' === get_class( $this ) ) ? 9 : 10 ;

		if ( isset( $admin_page_hooks['app-dashboard'] ) ) {
			add_action( "tabs_{$admin_page_hooks['app-dashboard']}_page_app-settings", array( $this, 'register' ), $priority );
			add_action( "tabs_{$admin_page_hooks['app-dashboard']}_page_app-settings_form_handler", array( $this, 'features_handler' ), $priority );
		}
	}

	/**
	 * Registers Listing settings
	 *
	 * @param APP_Tabs_Page $page Current settings page object.
	 */
	public function register( $page ) {

		$options = $this->listing->options;
		$type    = $this->listing->get_type();
		$name    = get_post_type_object( $type )->labels->name;

		$this->enabled_features = $this->listing->options->features;

		$page->tabs->add_after( 'general', $type, $name );
		$page->tabs->add( 'notifications', __( 'Notifications', APP_TD ) );

		$page->tab_sections[ $type ]['general'] = array(
			'title'  => __( 'General', APP_TD ),
			'fields' => array(),
			'options' => $options,
		);

		$page->tab_sections[ $type ]['moderation'] = array(
			'title'  => __( 'Moderation', APP_TD ),
			'fields' => array(
				array(
					'title' => __( 'Moderate Items', APP_TD ),
					'type'  => 'checkbox',
					'name'  => 'moderate',
					'desc'  => __( 'Manually approve and publish each new listing', APP_TD ),
					'tip'   => __( 'Left unchecked, listings go live immediately without being moderated (unless it has not been paid for).', APP_TD ),
				),
			),
			'options' => $options,
		);

		$page->tab_sections[ $type ]['permissions'] = array(
			'title'  => __( 'Permissions', APP_TD ),
			'fields' => array(
				array(
					'title' => __( 'Allow Edit Published Listings', APP_TD ),
					'type'  => 'checkbox',
					'name'  => 'allow_edit',
					'desc'  => __( 'Yes', APP_TD ),
					'tip'   => __( 'Do you allow to users edit their published listings? Note: this option has effect only if user is in Contributor role and lower, users in other roles can edit their listings anyway.', APP_TD ),
				),
				array(
					'title' => __( 'Allow Renew Expired Listings', APP_TD ),
					'type'  => 'checkbox',
					'name'  => 'allow_renew',
					'desc'  => __( 'Yes', APP_TD ),
					'tip'   => __( 'Do you allow to users to renew their expired listings? Renew listing process allows users to re-publish their expired listings by passing through all publishing steps again.', APP_TD ),
				),
			),
			'options' => $options,
		);

		$page->tab_sections[ $type ]['duration'] = array(
			'title'  => __( 'Duration', APP_TD ),
			'fields' => array(
				array(
					'title' => __( 'Items Duration', APP_TD ),
					'type'  => 'number',
					'name'  => 'duration',
					'tip'   => __( 'The duration in days to be applied to new items.', APP_TD ),
				),
			),
			'options' => $options,
		);

		$page->tab_sections[ $type ]['pricing'] = array(
			'title'  => __( 'Pricing', APP_TD ),
			'fields' => array(
				array(
					'title' => __( 'Charge for Items', APP_TD ),
					'name'  => 'charge',
					'type'  => 'checkbox',
					'desc'  => __( 'Start accepting payments', APP_TD ),
					'tip'   => sprintf( __( 'Do you want to charge for creating an item on your site? You can manage your <a href="%s">Payments Settings</a> in the Payments Menu.', APP_TD ), "admin.php?page=app-payments-settings&tab={$type}" ),
				),
			),
			'options' => $options,
		);

		$page->tab_sections[ $type ]['templates'] = array(
			'title'  => __( 'Templates', APP_TD ),
			'fields' => array(
				array(
					'title'    => __( 'Content Template', APP_TD ),
					'name'     => 'content_template',
					'type'     => 'custom',
					'render'   => array( 'APP_Editor_Field_Type', '_render' ),
					'sanitize' => 'wp_kses_post',
					'desc'     => '',
					'tip'      => sprintf( __( 'This is a content template with default markup. "%s" shortcode indicates where the actuall listing content will placed in relation to other elements.', APP_TD ), '[appthemes_listing_content]' ),
					'props'    => array(
						'editor_type' => 'tmce',
					),
				),
			),
			'options' => $options,
		);

		// Get all registered modules.
		$features = $this->listing->get_modules_raw();
		// Leave only optional modules.
		$features = wp_list_filter( $features, array( 'optional' => true ) );

		if ( ! empty( $features ) ) {
			$page->tab_sections[ $type ]['features'] = array(
				'title'  => __( 'Extra Features', APP_TD ),
				'fields' => array(
					array(
						'title'    => __( 'Activation', APP_TD ),
						'name'     => 'features',
						'type'     => 'checkbox',
						'choices'  => wp_list_pluck( $features, 'label' ),
						'tip'      => __( 'Enable Extra Features provided by custom addons for this Listing type.', APP_TD ),
					),
				),
				'options' => $options,
			);
		}

		$page->tab_sections['notifications'][ "{$type}_notifications" ] = array(
			'title'  => $name,
			'fields' => array(
				array(
					'title' => __( 'New Items', APP_TD ),
					'type'  => 'checkbox',
					'name'  => "notify_new_$type",
					'desc'  => __( 'Yes', APP_TD ),
					'tip'   => __( 'Notify admins when new items are posted', APP_TD ),
				),
				array(
					'title' => __( 'Pending Items', APP_TD ),
					'type'  => 'checkbox',
					'name'  => "notify_pending_$type",
					'desc'  => __( 'Yes', APP_TD ),
					'tip'   => __( 'Notify admins when new items are waiting for moderation', APP_TD ),
				),
				array(
					'title' => __( 'Renew Items', APP_TD ),
					'type'  => 'checkbox',
					'name'  => "notify_renew_$type",
					'desc'  => __( 'Yes', APP_TD ),
					'tip'   => __( 'Notify admins when items are renewed', APP_TD ),
				),
			),
			'options' => $options,
		);
	}

	function features_handler( $page ) {
		$updated_features = $this->listing->options->features;

		if ( $this->enabled_features !== $updated_features ) {
			// Otherwise update page to enable updated features.
			$url = add_query_arg( 'firstrun', 1 );
			$url = esc_url_raw( $url );
			wp_redirect( $url );
			appthemes_exit();
		}
	}
}
