<?php
/**
 * Post Type Listing Builder
 *
 * @package Listing\Builder
 * @author  AppThemes
 * @since   Listing 1.0
 */

/**
 * A generic Listing Builder to construct a Listing object associated with
 * already registered post type (like 'post' or 'attachment').
 *
 * Supposed to be a base builder class for any Listing types that depended on
 * custom post types.
 */
class APP_Post_Listing_Builder extends APP_Listing_Builder {

	/**
	 * Creates empty Listing instance.
	 *
	 * @param string $type The Listing type.
	 */
	public function create( $type = '' ) {

		if ( ! $type ) {
			$type = 'post';
		}

		$this->modules = new APP_Listing_Dependencies();
		$this->listing = new APP_Listing( $type, $this->modules );
	}

	/**
	 * Registers listing stuff in the WordPress system (i.e. taxonomies, post
	 * types, comment types, etc.).
	 */
	public function register() {
		$this->_register_taxonomies();
		$this->_register_listing_type();
	}

	/**
	 * Sets up basic listing options.
	 */
	public function set_options() {
		$options = new APP_Listing_Options( $this->listing );
		$this->listing->set_options( $options );
	}

	/**
	 * Registers taxonomies associated with current listing type
	 */
	protected function _register_taxonomies() {}

	/**
	 * Registers Listing type as post type
	 */
	protected function _register_listing_type() {}

	/**
	 * Configure Listing instance with essential modules included in a
	 * listing package.
	 */
	public function add_modules() {

		if ( $this->listing->options->charge ) {
			$this->listing->add_module( 'payments', 'APP_Listing_Payments' );
		}

		$this->listing->add_module( 'plans',                 'APP_Listing_Plans_Registry' );
		$this->listing->add_module( 'view_process_new',      'APP_View_Process_New' );
		$this->listing->add_module( 'view_process_edit',     'APP_View_Process_Edit' );
		$this->listing->add_module( 'view_process_renew',    'APP_View_Process_Renew' );
		$this->listing->add_module( 'view_process_upgrade',  'APP_View_Process_Upgrade' );
		$this->listing->add_module( 'step_select_plan',      'APP_Step_Select_Plan' );
		$this->listing->add_module( 'step_current_plan',     'APP_Step_Current_Plan' );
		$this->listing->add_module( 'step_activate',         'APP_Step_Activate' );
		$this->listing->add_module( 'step_upgrade',          'APP_Step_Upgrade' );
		$this->listing->add_module( 'step_moderate',         'APP_Step_Moderate' );
		$this->listing->add_module( 'step_activate_order',   'APP_Step_Activate_Order', array( 'payments' ) );
		$this->listing->add_module( 'step_edit_info',        'APP_Step_Edit_Info' );
		$this->listing->add_module( 'step_edit_info_new',    'APP_Step_Edit_Info_New' );
		$this->listing->add_module( 'step_edit_info_optional', 'APP_Step_Edit_Info_Optional' );
		$this->listing->add_module( 'step_create_order',     'APP_Step_Create_Order', array( 'payments' ) );
		$this->listing->add_module( 'step_upgrade_order',    'APP_Step_Create_Upgrade_Order', array( 'payments' ) );
		$this->listing->add_module( 'step_gateway_select',   'APP_Step_Gateway_Select', array( 'payments' ) );
		$this->listing->add_module( 'step_gateway_process',  'APP_Step_Gateway_Process', array( 'payments' ) );
		$this->listing->add_module( 'expire',                'APP_Listing_Expire' );
		$this->listing->add_module( 'form',                  'APP_Listing_Form' );
		$this->listing->add_module( 'caps',                  'APP_Listing_Caps' );
		$this->listing->add_module( 'details',               'APP_Listing_Details' );
		$this->listing->add_module( 'process_widget',        'APP_Widget_Process_Listing_Button' );

		if ( is_admin() ) {
			$this->listing->add_module( 'settings',          'APP_Listing_Settings' );
			$this->listing->add_module( 'admin_security',    'APP_Listing_Admin_Security' );
		}

		// Optional features.
		$this->listing->add_module(
			'multi_plans',
			'APP_Listing_Plans',
			array( 'plans' ),
			array(),
			array(
				'optional' => true,
				'label'    => __( 'Multiple Listing Plans', APP_TD ),
			)
		);

		$this->listing->add_module(
			'addons',
			'APP_Listing_Addons',
			array( 'payments' ),
			array(),
			array(
				'optional' => true,
				'label'    => __( 'Listing Pricing Addons', APP_TD ),
			)
		);

		$this->listing->add_module(
			'recurring',
			'APP_Listing_Recurring',
			array( 'payments' ),
			array(),
			array(
				'optional' => true,
				'label'    => __( 'Recurring Payments', APP_TD ),
			)
		);

		// Following taxonomy-depended modules must have dynamic names, i.e "{$taxonomy}_limit".
		$this->listing->add_module(
			'category_limit',
			'APP_Listing_Taxonomy_Limit',
			array(),
			array( 'category' ),
			array(
				'optional' => true,
				'label'    => __( 'Category Limit', APP_TD ),
			)
		);

		$this->listing->add_module(
			'category_surcharge',
			'APP_Listing_Taxonomy_Surcharge',
			array( 'payments' ),
			array( 'category' ),
			array(
				'optional' => true,
				'label'    => __( 'Category Surcharges', APP_TD ),
			)
		);

		$this->listing->add_module(
			'custom_form',
			'APP_Listing_Custom_Forms',
			array( 'form' ),
			array( 'category' ),
			array(
				'optional' => true,
				'label'    => __( 'Custom Forms', APP_TD ),
			)
		);

		$this->listing->add_module(
			'claim',
			'APP_Listing_Claim',
			array(),
			array(),
			array(
				'optional' => true,
				'label'    => __( 'Claim Listings', APP_TD ),
			)
		);

	}

	/**
	 * Enqueues a module classes in a Listing's dependency object.
	 *
	 * Filters disabled modules. Only enqueued modules will be instatiated.
	 *
	 * @param mixed $handles Item handle and argument (string) or item handles
	 *                        and arguments (array of strings). If argument is
	 *                        false, all registered items will be enqueued.
	 */
	public function enqueue_modules( $handles = false ) {

		if ( false === $handles ) {
			$handles = array_keys( $this->modules->registered );
		}

		$allowed = (array) $this->listing->options->features;

		foreach ( (array) $handles as $handle ) {
			if ( ! $this->modules->get_data( $handle, 'optional' ) || in_array( $handle, $allowed ) ) {
				$this->modules->enqueue( $handle );
			}
		}
	}

	/**
	 * Instatiates registered submodules.
	 *
	 * Processes the items passed to it or the queue, and their dependencies.
	 *
	 * @param mixed $handles Optional. Items to be processed:
	 *                        Process queue (false),
	 *                        process item (string),
	 *                        process items (array of strings).
	 */
	public function set_modules( $handles = false ) {
		$this->modules->do_items( $handles );
	}

}
