<?php
/**
 * Listing Form submodule
 *
 * @package Listing\Form
 * @author  AppThemes
 * @since   Listing 1.0
 */

/**
 * Form processing class
 */
class APP_Listing_Form {

	/**
	 * Current Listing module object.
	 *
	 * @var APP_Listing
	 */
	protected $listing;

	/**
	 * Errors object.
	 *
	 * @var WP_Error
	 */
	protected $errors;

	/**
	 * Cached raw form fields array.
	 *
	 * @var array
	 */
	private $form_raw = array();

	/**
	 * Module settings object.
	 *
	 * @var APP_Listing_Form_Settings
	 */
	protected $settings;

	/**
	 * Single Listing Fields metabox object.
	 *
	 * @var APP_Listing_Fields_Metabox
	 */
	protected $fields_box;

	/**
	 * Single Listing Media Fields metabox object.
	 *
	 * @var APP_Listing_Media_Metabox
	 */
	protected $media_box;

	/**
	 * Constructs listing form object
	 *
	 * @param APP_Listing $listing Listing object to assign process with.
	 */
	public function __construct( APP_Listing $listing ) {

		$this->listing = $listing;
		$this->errors  = new WP_Error();
		$this->listing->options->set_defaults( $this->get_defaults() );

		if ( did_action( 'init' ) ) {
			$this->init();
		} else {
			add_action( 'init', array( $this, 'init' ) );
		}
	}

	/**
	 * Init module
	 */
	public function init() {
		$this->activate_modules();
	}

	/**
	 * Activates submodules
	 */
	protected function activate_modules() {
		if ( ! $this->settings && is_admin() ) {
			$this->settings = new APP_Listing_Form_Settings( $this->listing );
		}

		if ( ! $this->fields_box && is_admin() ) {
			$this->fields_box = new APP_Listing_Fields_Metabox( $this->listing );
		}

		if ( ! $this->media_box && is_admin() ) {
			$this->media_box = new APP_Listing_Media_Metabox( $this->listing );
		}

	}

	/**
	 * Retrieves core fields
	 *
	 * @return array An array of the fields parameters
	 */
	public function get_core_fields() {

		return apply_filters( 'appthemes_core_fields', array(
			array(
				'type'  => 'input_text',
				'id'    => 'post_title',
				'props' => array(
					'required' => 1,
					'label'    => __( 'Title', APP_TD ),
				),
			),
			array(
				'type'  => 'textarea',
				'id'    => 'post_content',
				'props' => array(
					'required'    => 1,
					'label'       => __( 'Description', APP_TD ),
					'editor_type' => '',
				),
			),
		), $this->listing->get_type() );
	}

	/**
	 * Retrieves module's options default values to be registered in Listing
	 * options object.
	 *
	 * @return array The Form defaults.
	 */
	public function get_defaults() {
		return array( 'app_form' => $this->get_core_fields() );
	}

	/**
	 * Get form fields metadata retrieved from the listing item.
	 *
	 * @param WP_Post $listing Listing object to get data from.
	 *
	 * @return array Form data
	 */
	public function get_formdata( $listing ) {
		$formdata = array();

		// Get form field values if it stored somewhere.
		$custom_fields = get_post_custom( $listing->ID );

		foreach ( $custom_fields as &$value ) {
			$value = maybe_unserialize( $value[0] );
		}

		$formdata = array_merge( $custom_fields, $listing->to_array() );

		$taxonomies = get_object_taxonomies( $this->listing->get_type() );

		foreach ( $taxonomies as $taxonomy ) {
			$formdata['tax_input'][ $taxonomy ] = wp_list_pluck( wp_get_object_terms( $listing->ID, $taxonomy ), 'name', 'term_id' );
		}

		return $formdata;
	}

	/**
	 * Retrieves a raw array of the form fields
	 *
	 * @return array Raw form fields
	 */
	public final function get_form_fields_raw() {

		if ( empty( $this->form_raw ) ) {
			$form_raw = $this->listing->options->app_form;

			if ( empty( $form_raw ) ) {
				$form_raw = $this->get_core_fields();
			}

			$this->form_raw = APP_Form_Builder::prepare_fields( $form_raw );
		}

		return $this->form_raw;
	}

	/**
	 * Retrieves an array of the fields after include/exclude filters to be used
	 * for the form processing or display.
	 *
	 * @param array $exclude_filters An associative array with field
	 *                               attribute/value(s) used to excluded certain
	 *                               fields from the final field list
	 *                               (e.g: array( 'name' => 'my-field' ) ).
	 * @param array $include_filters Retrieves A list of field attributes used
	 *                               to include certain fields in the final
	 *                               field list (similar to $exclude_filters).
	 * @param int   $item_id        Current Listing item ID.
	 *
	 * @return array Form fields.
	 */
	public function get_form_fields( $exclude_filters = array(), $include_filters = array(), $item_id = 0 ) {

		$form = $this->get_form_fields_raw();

		foreach ( $form as $key => &$field ) {

			// Remove field if it is disabled or excluded or not included.
			if ( ! empty( $field['props']['disable'] ) || ! $this->filter( $field, $exclude_filters, $include = false ) || ! $this->filter( $field, $include_filters ) ) {
				unset( $form[ $key ] );
				continue;
			}

			$field = $this->apply_atts( $field, $item_id );
			$field = apply_filters( 'appthemes_form_field', $field, $item_id );

			if ( ! $field ) {
				unset( $form[ $key ] );
			}
		}

		return $form;
	}

	public function apply_atts( $field, $item_id ) {
		// Wrap radios and checkbox's using unordered lists tags to make them consistent with WP walkers markup.
		if ( 'radio' === $field['type'] || 'checkbox' === $field['type'] ) {
			$field['wrap'] = '<ul>%input%</ul>';
			$field['wrap_each'] = '<li>%input%</li>';
		} else if ( 'tax_input' === $field['type'] ) {
			$field['name']   = array( 'tax_input', $field['props']['tax'] );
			$field['render'] = array( 'APP_Tax_Input_Field_Type', '_render' );
		} else if ( 'textarea' === $field['type'] && isset( $field['props']['editor_type'] ) && $field['props']['editor_type'] ) {
			$field['render'] = array( 'APP_Editor_Field_Type', '_render' );
		} else if ( 'file' === $field['type'] ) {
			$field['render'] = array( 'APP_Media_Field_Type', '_render' );
		}

		$field['title'] = $field['desc'];
		unset( $field['desc'] );

		// Preserve initial field type.
		$field['props']['type'] = $field['type'];

		return $field;
	}

	/**
	 * Retrieves generated form HTML
	 *
	 * @param WP_Post $listing     Current listing object.
	 * @param array   $form_fields Form fields.
	 *
	 * @return string Generated HTML
	 */
	public function get_form( WP_Post $listing, $form_fields = array() ) {

		if ( empty( $form_fields ) ) {
			$form_fields = $this->get_form_fields( array(), array(), $listing->ID );
		}

		$output   = '';
		$formdata = $this->get_formdata( $listing );

		// Generate fields html.
		foreach ( $form_fields as $field ) {

			$defaults = array(
				// Might be necessary for rendering.
				'listing_id' => $listing->ID,
				'wrap'       => scbForms::TOKEN,
			);

			$field = wp_parse_args( $field, $defaults );

			$field['wrap'] = $this->field_wrap( $field['wrap'], $field );

			// Set renderer (by default initial ones).
			if ( ! isset( $field['render'] ) ) {
				$field['render'] = array( scbFormField::create( $field ), 'render' );
			}

			$type = $field['type'];

			// Set 'custom' field type.
			$field['type'] = 'custom';

			$html = $this->field_render( $field, $formdata );

			// Temporary fix scbForms::input() since it doesn't take 'extra parameter for radios and checkboxes'
			// TODO: Provide a smart solution for this fix.
			if ( ( 'radio' === $type || 'checkbox' === $type )
				&& isset( $field['extra'] )
				&& isset( $field['extra']['class'] )
				&& 'required' === $field['extra']['class']
			) {
				$html = str_replace( 'type=', 'class="required" type=', $html );
			}
			$output .= apply_filters( 'appthemes_render_form_field', $html, $field, $listing->ID );
		}

		return $output;
	}

	/**
	 * Generates field html.
	 *
	 * @uses scbForms::input()
	 *
	 * @param array $field    Field parameters to be used in scbForms.
	 * @param array $formdata Field value.
	 *
	 * @return string Generated field html.
	 */
	public function field_render( $field, $formdata = null ) {
		return scbForms::input( $field, $formdata );
	}

	/**
	 * Wraps form field token and adds label and description.
	 *
	 * @param array $token Field input token to be replaced with generated html.
	 * @param array $field Field parameters.
	 *
	 * @return string Wrapped field html.
	 */
	public function field_wrap( $token, $field ) {
		$label = html( 'label', array( 'for' => scbForms::get_name( $field['name'] ) ), $field['title'] );
		$type  = $field['props']['type'];

		return html( 'div class="form-field ' . $type . '"', $label . $token );
	}

	/**
	 * Processes listing form
	 *
	 * @param int    $listing_id  Listing ID.
	 * @param string $action      Action nonce.
	 * @param string $name        Nonce name.
	 * @param array  $form_fields Form fields.
	 *
	 * @return int Processed listing ID
	 */
	public function process_form( $listing_id, $action, $name, $form_fields = array() ) {

		check_admin_referer( $action, $name );

		if ( empty( $form_fields ) ) {
			$form_fields = $this->get_form_fields( array(), array(), $listing_id );
		}

		/**
		 * Allows to add or remove fields withing processing method.
		 */
		$form_fields = apply_filters( 'appthemes_process_listing_form_fields', $form_fields, $listing_id );

		if ( empty( $form_fields ) ) {
			return $listing_id;
		}

		$field_names = wp_list_pluck( $form_fields, 'name' );

		foreach ( $field_names as $key => $field_name ) {
			if ( is_array( $field_name ) ) {
				$field_names[ $key ] = array_shift( $field_name );
			}
		}

		$field_names = array_unique( $field_names );
		$formdata    = wp_array_slice_assoc( wp_unslash( $_POST ), $field_names ); // Input var okay.
		$formdata    = $this->validate_post_data( $form_fields, $formdata );

		if ( $formdata instanceof WP_Error && $formdata->get_error_codes() ) {
			return $this->errors;
		}

		$core_fields = array(
			'post_title',
			'post_content',
			'tax_input',
		);

		$postarr = wp_array_slice_assoc( $formdata, $core_fields );

		if ( ! empty( $postarr ) ) {
			$postarr['ID']        = $listing_id;
			$postarr['post_type'] = $this->listing->get_type();
			$listing_id           = wp_update_post( $postarr, true );
		}

		if ( is_wp_error( $listing_id ) ) {
			return $listing_id;
		}

		$meta_fields = wp_array_slice_assoc( $formdata, array_diff( array_keys( $formdata ), $core_fields ) );

		foreach ( $meta_fields as $meta_field => $value ) {
			update_post_meta( $listing_id, $meta_field, $value );
		}

		appthemes_handle_media_upload( $listing_id, array(), true );

		$listing_id = apply_filters( 'appthemes_handle_listing_form', $listing_id, $form_fields, $formdata, $this->errors );

		if ( $this->errors->get_error_codes() ) {
			$listing_id = $this->errors;
		}

		return $listing_id;
	}

	/**
	 * Sanitizes and validates post data.
	 *
	 * @param array $form_fields Form fields to be validated.
	 * @param array $formdata    Posted data.
	 *
	 * @return array|WP_Error Validated post data or error object.
	 */
	public function validate_post_data( $form_fields, $formdata ) {

		$validator = new APP_Form_Field_Validator( $this->errors );

		foreach ( $form_fields as &$field ) {

			$scbField = scbFormField::create( $field );

			// 'sanitizers' item conteins numeric array of the validation methods
			// first item is native scbForm field validate method, others custom
			$sanitizers = array();

			if ( 'textarea' === $field['type']  && isset( $field['props']['editor_type'] ) && $field['props']['editor_type'] ) {
				$sanitizers[] = 'wp_kses_post';
			} else {
				$sanitizers[] = array( $scbField, 'validate' );
			}

			// Validate required fields (common method).
			$sanitizers[] = array( $validator, 'required' );

			// Set general sanitize/validate method which calls all sanitizers.
			$field['sanitize'] = array( $validator, 'validate' );

			if ( isset( $field['sanitizers'] ) && is_array( $field['sanitizers'] ) ) {
				$sanitizers = array_merge( $sanitizers, $field['sanitizers'] );
			}

			/**
			 * Allows to add extra sanitize/validate methods.
			 */
			$sanitizers = apply_filters( 'appthemes_form_field_validators', $sanitizers, $field, $this->listing->get_type() );

			$field['sanitizers'] = $sanitizers;

			// Preserve initial field type.
			$field['props']['type'] = $field['type'];
			// Set 'custom' field type.
			$field['type'] = 'custom';
		}

		$formdata = scbForms::validate_post_data( $form_fields, $formdata, $formdata );

		if ( $this->errors->get_error_codes() ) {
			$formdata = $this->errors;
		}

		return $formdata;
	}

	/**
	 * Helper function to filter out a field based on its attributes.
	 *
	 * @param  array   $field   The field array.
	 * @param  array   $filters The filter array with key/value pairs of field
	 *                          attributes (e.g: 'name' => 'my-custom-field' ).
	 * @param  boolean $include Whether to include or exclude field by given
	 *                          filters.
	 *
	 * @return boolean Retrieves the boolean value set in '$include' if the
	 *                 filter condition is met, opposite value otherwise.
	 *                 Returns TRUE, if empty filters.
	 */
	public final function filter( $field, $filters, $include = true ) {

		if ( empty( $filters ) ) {
			return true;
		}

		// Exclude fields based on certain criteria.
		foreach ( (array) $filters as $att => $values ) {

			if ( isset( $field[ $att ] ) && in_array( $field[ $att ], (array) $values ) ) {
				return $include;
			}
		}

		return ! $include;
	}
}
