<?php
/**
 * Favorites functionality.
 *
 * @package Vantage\Favorites
 * @author AppThemes
 * @since 2.0.0
 */

/**
 * Initializes favorites ajax listeners.
 *
 * @since 2.0.0
 *
 * @return void
 */
function va_favorites_init() {

	p2p_register_connection_type( array(
		'name' => VA_LISTING_FAVORITES,
		'from' => VA_LISTING_PTYPE,
		'to'   => 'user',
	) );

	add_action( 'wp_ajax_vantage_favorites', 'va_handle_ajax_favorites' );
	add_action( 'wp_ajax_nopriv_vantage_favorites', 'va_handle_ajax_favorites' );
}
add_action( 'init', 'va_favorites_init', 13 );

/**
 * Ajax handler for favoriting an item.
 *
 * @since 2.0.0
 * @since 4.0.0 Rebuilt so it's more robust.
 *
 * @return void
 */
function va_handle_ajax_favorites() {

	// We need certain params to proceed.
	if ( ! isset( $_POST[ 'type' ] ) && ! isset( $_POST[ 'id' ] ) ) {
		wp_die( -1 );
	}

	// Should be handled on page load. Extra check here.
	if ( ! is_user_logged_in() ) {
		wp_die( -1 );
	}

	$listing_id = (int) $_POST[ 'id' ];

	// Make sure it's a legit request.
	check_ajax_referer( 'favorite-' . $listing_id, 'nonce' );

	// Process the new favorite.
	if ( 'add' == $_POST[ 'type' ] ) {

		// Get the text strings we need.
		$response = va_fave_text_strings();
		$response = $response[ 'delete' ];

		// Save the favorite in the db.
		$p2p = p2p_type( VA_LISTING_FAVORITES )->connect( $listing_id, get_current_user_id(), array( 'date' => current_time( 'mysql' ) ) );
	} else {

		// Get the text strings we need.
		$response = va_fave_text_strings();
		$response = $response[ 'add' ];

		// Delete the favorite from the db.
		$p2p = p2p_type( VA_LISTING_FAVORITES )->disconnect( $listing_id, get_current_user_id() );
	}

	// Handle the error.
	if ( is_wp_error( $p2p ) ) {
		wp_send_json_error( __( 'P2P Error. Could not add favorite.', APP_TD ) );
	}

	// Return the response.
	wp_send_json_success( $response );
}

/**
 * Check if a specific listing is already favorited.
 *
 * @param int $listing_id The listing id to search in
 * @return bool Returns True if already favorited, False otherwise
 */
function va_is_fave_listing( $listing_id ) {

	$count = p2p_get_connections( VA_LISTING_FAVORITES, array(
		'direction' => 'from',
		'from'      => $listing_id,
		'to'        => get_current_user_id(),
		'fields'    => 'count',
	) );

	return (bool) $count;
}

/**
 * Returns or echoes the favorite button.
 *
 * @since 3.0.0
 *
 * @return string
 */
function the_listing_faves_link() {
	global $post;

	// Get the text strings we need.
	$text = va_fave_text_strings();

	if ( ! is_user_logged_in() ) {
		return '<span data-tooltip aria-haspopup="true" class="has-tip top listing-icon fave-button" title="' .  esc_attr( $text[ 'errors' ][ 'not_logged_in' ] ) . '"><i class="' . esc_attr( $text[ 'add' ][ 'icon' ] ) . '"></i><span class="fave-text">' .  $text[ 'add' ][ 'text' ] . '</span></span>';
	}

	$nonce = wp_create_nonce( 'favorite-' . $post->ID );

	if ( ! va_is_fave_listing( $post->ID ) ) {
		$text = $text[ 'add' ];
	} else {
		$text = $text[ 'delete' ];
	}

	return '<a class="listing-icon fave-button" href="" data-id="' . esc_attr( $post->ID ) . '" data-type="' . esc_attr( $text[ 'type' ] ) . '" data-nonce="' . esc_attr( $nonce ) . '" rel="nofollow"><i class="' . esc_attr( $text[ 'icon' ] ) . '" title="' . esc_attr( $text[ 'text' ] ) . '"></i><span class="fave-text">' . $text[ 'text' ] . '</span></a>';
}
add_action( 'va_listing_item_head', 'the_listing_faves_link' );

/**
 * The text strings used on the front end.
 *
 * @since 4.0.0
 *
 * @return array
 */
function va_fave_text_strings() {

	$text_strings = array();

	$text_strings['errors'] = array(
		'not_logged_in' => __( 'You must be logged in to save favorites.', APP_TD ),
	);

	$text_strings['add'] = array(
		'text' => _x( 'Favorite', 'text for ajax link', APP_TD ),
		'type' => 'add',
		'icon' => 'fa fa-heart-o',
	);

	$text_strings['delete'] = array(
		'text' => _x( 'Unfavorite', 'text for ajax link', APP_TD ),
		'type' => 'delete',
		'icon' => 'fa fa-heart',
	);

	/**
	 * Filter the favorites text string array.
	 *
	 * @since 4.0.0
	 *
	 * @param array $text_strings Arrays of text strings used in the favorite ajax feature.
	 */
	return apply_filters( 'va_fave_text_strings', $text_strings );
}

/**
 * Displays favorite link on single listing page.
 *
 * @since 4.0.0
 *
 * @global WP_Post $post Global $post.
 *
 * @return void
 */
function va_fave_listing_link() {
	global $post;

	/**
	 * Filter the fave listing link.
	 *
	 * @since 4.0.0
	 *
	 * @param string $listing_faves_link The listing favorities link.
	 * @param object $post The global post object.
	 */
 	echo apply_filters( 'va_fave_listing_link', the_listing_faves_link(), $post );
}
add_action( 'va_listing_single_bar_actions', 'va_fave_listing_link', 20 );
