<?php

/**
 * Queries the database for attachments from custom fields or gallery
 * Uses the meta key '_va_attachment_type' to filter the available attachment types: gallery | file
 *
 * @param int  	  $post_id The post ID
 * @param int     $how_many (optional) The number of attachments to retrieve
 * @param string  $type     (optional) The type of attachment to return (gallery|file)
 * @param string  $fields   (optional) The fields to be returned
 *
 * @return array
 */
function va_get_post_attachments( $post_id, $how_many = -1, $type = VA_ATTACHMENT_GALLERY, $fields = '' ) {
	if ( ! $post_id ) {
		return array();
	}

	// In case if featured attachement has different post parent
	$feat_attachement = array();

	if ( VA_ATTACHMENT_GALLERY == $type ) {
			// display gallery images
			$meta_query = array(
				'meta_query' => array(
					'relation' => 'OR',
					array(
						'key'     => '_va_attachment_type',
						'value'   => VA_ATTACHMENT_FILE,
						'compare'	=> '!=',
					),
					array(
						'key'     => '_va_attachment_type',
						'value'   => 'dummy', // added dummy text since empty string or NULL won't work with 'NOT EXISTS'
						'compare'	=> 'NOT EXISTS'
					),
				),
			);

			$featured_id = get_post_thumbnail_id( $post_id );

			if ( $featured_id && ! is_wp_error( $featured_id ) ) {
				$feat_post = get_post( $featured_id );
				if ( $feat_post->post_parent != $post_id ) {
					$feat_attachement[] = $feat_post;
				}
			}

	} else {
			// display file attachments only
			$meta_query = array(
					'meta_key'		=> '_va_attachment_type',
					'meta_value'	=> $type,
			);

	}

	$args = array(
		'post_type'      => 'attachment',
		'post_status'    => 'inherit',
		'post_parent'    => $post_id,
		'posts_per_page' => $how_many,
		'orderby'        => 'menu_order',
		'order'          => 'asc',
		'fields'         => $fields,
		'update_post_term_cache' => false,
	);
	$args = array_merge( $args, $meta_query );

	$results = get_posts( $args );

	return array_merge( $feat_attachement, $results );
}


/**
 * Collects and returns the meta info for a specific attachment ID.
 *
 * @param int $attachment_id The attachment ID
 *
 * @return array Returns the attachment meta
 */
function va_get_file_meta( $attachment_id ) {
	$filename = wp_get_attachment_url( $attachment_id );

	$title = trim( strip_tags( get_the_title( $attachment_id ) ) );
	$basename = basename( $filename );
	$size = size_format( filesize( get_attached_file( $attachment_id ) ), 2 );

	$meta = array(
		'title'     => ( ! $title ? $basename : $title ),
		'url'       => $filename,
		'mime_type' => get_post_mime_type( $attachment_id ),
		'size'      => $size,
	);
	return $meta;
}


/**
 * Returns the file link for a specific attachment ID.
 *
 * @param int $attachment_id The attachment ID
 *
 * @return string Returns the HTML link
 */
function va_get_file_link( $attachment_id ) {

	$file = va_get_file_meta( $attachment_id );

	$link = html( 'a', array(
		'href' => $file['url'],
		'title' => $file['title'],
		'target' => '_blank',
	), $file['title'] );

	return html( 'div', array(
		'class' => 'file-extension ' . va_get_mime_type_icon( $file['mime_type'] ),
	), $link );
}

/**
 * Returns the related icon filename for a mime-type
 *
 * @param string  $mime_type
 *
 * @return string  Returns the mime type icon file name
 */
function va_get_mime_type_icon( $mime_type ) {

	if ( ! $mime_type ) {
		$mime_type = 'generic';
	}

	$file_ext_ico = array (
		'pdf'          => 'file-pdf',
		'msword'       => 'file-word',
		'vnd.ms-excel' => 'file-excel',
		'csv'          => 'file-excel',
		'image'        => 'file-image',
		'other'        => 'file-other',
	);

	$mime_type = explode( '/' , $mime_type );
	if ( is_array( $mime_type ) ) {
		// simplify the mime match for image types by using the 'image' part (i.e: image/png, image/jpg, etc)
		$mime_type = ( 'image' == $mime_type[0] ? $mime_type[0] : $mime_type[1] );
	}

	if ( ! isset( $file_ext_ico[ $mime_type ] ) ) {
		$mime_type = 'other';
	}

	return apply_filters( 'va_mime_type_icon', $file_ext_ico[ $mime_type ], $mime_type );
}

/**
 * Displays the files list for a listing/event/post_type
 *
 * @return void
 */
function va_the_files_list() {
	$post_id = get_the_ID();
	$post_type = get_post_type( $post_id );
	$post_type_obj = get_post_type_object( $post_type );

	$attachments = va_get_post_attachments( $post_id, -1, VA_ATTACHMENT_FILE );

	if ( empty( $attachments ) ) {
		return;
	}

	echo '<section id="' . $post_type . '-files">';

	echo '<div class="' . $post_type . '-files">';

	echo sprintf( __( '%s Attachments:', APP_TD ), $post_type_obj->labels->singular_name );


	foreach ( $attachments as $attachment ) {
		echo va_get_file_link( $attachment->ID );
	}

	echo '</div>';

	echo '</section>';

}
