<?php
/**
 * Theme helper functions.
 *
 * @package Vantage
 * @since   4.0.0
 */

/**
 * Rename the WordPress sticky class since it conflicts with Foundation.
 *
 * @since 4.0.0
 *
 * @param array $classes Array of post css classes.
 * @return array         Array of post css classes.
 */
function va_rename_sticky_class( $classes ) {

	$classes   = array_diff( $classes, array( 'sticky' ) );
	$classes[] = 'wp-sticky';

	return $classes;
}
add_filter( 'post_class', 'va_rename_sticky_class' );

/**
 * Displays an optional post thumbnail.
 *
 * @since 4.0.0
 */
function va_post_thumbnail() {
	if ( post_password_required() || is_attachment() || ! has_post_thumbnail() ) {
		return;
	}
?>
<figure class="entry-thumbnail">
	<?php
	if ( is_singular( array( 'post', VA_LISTING_PTYPE ) ) ) :
		the_post_thumbnail( 'full' );
	else :
	?>
		<a class="entry-thumbnail" href="<?php the_permalink(); ?>" aria-hidden="true">
			<?php the_post_thumbnail( 'entry-thumbnail', array( 'alt' => the_title_attribute( 'echo=0' ) ) ); ?>
		</a>
	</figure><!-- .post-thumbnail -->

	<?php endif;
}

/**
 * Post pagination for Foundation support.
 *
 * @since  4.0.0
 *
 * @param  WP_Query $wp_query The WP_Query instance.
 * @return string The pagination results.
 */
function va_the_pagination( $wp_query = null ) {

	if ( is_null( $wp_query ) ) {
		$wp_query = $GLOBALS['wp_query'];
	}

	if ( 1 == $wp_query->max_num_pages ) {
		return;
	}

	$page_links = paginate_links( apply_filters( 'job_manager_pagination_args', array(
		'base'      => esc_url_raw( str_replace( 999999999, '%#%', get_pagenum_link( 999999999, false ) ) ),
		'format'    => '',
		'current'   => max( 1, get_query_var( 'paged' ) ),
		'total'     => $wp_query->max_num_pages,
		'prev_text' => '&larr;',
		'next_text' => '&rarr;',
		'type'      => 'list',
		'end_size'  => 3,
		'mid_size'  => 3,
	) ) );


	// Not pretty but we need to replace default WP markup with Foundation.
	$r = str_replace( "<ul class='page-numbers'>", "<ul class='pagination text-center' role='navigation'>", $page_links );
	$r = str_replace( '<li><span class="page-numbers dots">', "<li><a href='#'>", $r );
	$r = str_replace( "<li><span class='page-numbers current'>", "<li class='current'><a href='#'>", $r );
	$r = str_replace( '</span>', '</a>', $r );
	$r = str_replace( "<li><a href='#'>&hellip;</a></li>", "<li><span class='dots'>&hellip;</span></li>", $r );

	echo $r;
}

/**
 * Count the number of footer sidebars to enable dynamic classes for the footer.
 *
 * @since 4.0.0
 */
function va_footer_sidebar_class() {
	$count = 0;

	if ( is_active_sidebar( 'va-footer-sidebar-1' ) ) {
		$count++;
	}

	if ( is_active_sidebar( 'va-footer-sidebar-2' ) ) {
		$count++;
	}

	if ( is_active_sidebar( 'va-footer-sidebar-3' ) ) {
		$count++;
	}

	if ( is_active_sidebar( 'va-footer-sidebar-4' ) ) {
		$count++;
	}

	$class = '';

	switch ( $count ) {
		case '1':
			$class = 'large-12';
			break;
		case '2':
			$class = 'medium-6 large-6';
			break;
		case '3':
			$class = 'medium-6 large-4';
			break;
		case '4':
			$class = 'medium-6 large-3';
			break;
	}

	if ( $class ) {
		return $class;
	}
}

/**
 * Figure out the number of grid elements to use based on blog sidebar presence.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_blog_single_class() {

	if ( 'none' != get_theme_mod( 'blog_sidebar_position', 'right' ) ) {
		return 'medium-8';
	} else {
		return 'medium-10 medium-centered';
	}
}

/**
 * Figure out the number of grid elements to use based on listing sidebar presence.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_listing_single_class() {

	if ( 'none' != get_theme_mod( 'listing_sidebar_position', 'right' ) ) {
		return 'medium-8';
	} else {
		return 'medium-12';
	}
}

/**
 * Determines the maximum user upload size allowed.
 *
 * The default uses the max set in php.ini which is the ceiling limit.
 * If you wish to reduce it further, use the filter to set it in bytes.
 *
 * @since 4.0.0
 *
 * @return string $atts
 */
function va_upload_size_limit() {

	$u_bytes = wp_convert_hr_to_bytes( ini_get( 'upload_max_filesize' ) );
	$p_bytes = wp_convert_hr_to_bytes( ini_get( 'post_max_size' ) );

	/**
	 * Filters the maximum upload size allowed in php.ini.
	 * Use this if you want to restrict size limits for front-end user uploads.
	 *
	 * @since 4.0.0
	 *
	 * @param int $size    Max upload size limit in bytes.
	 * @param int $u_bytes Maximum upload filesize in bytes.
	 * @param int $p_bytes Maximum size of POST data in bytes.
	 */
	return apply_filters( 'va_upload_size_limit', min( $u_bytes, $p_bytes ), $u_bytes, $p_bytes );
}

/**
 * Returns a clean human-readable max file upload size limit number.
 *
 * @since 4.0.0
 *
 * @return string $atts
 */
function va_max_upload_size() {

	$max_upload_size = va_upload_size_limit();

	return size_format( (int) $max_upload_size );
}

/**
 * Displays a background cover image for pages, listings, and posts.
 *
 * @since 4.0.0
 *
 * @param string $class Pass in a css class.
 * @param array  $args Pass in some arguments.
 * @return string $atts
 */
function va_background_cover( $class, $args = array() ) {
	global $post, $wp_query;

	/**
	 * Filters the background cover photo default values.
	 *
	 * @since 4.0.0
	 */
	$defaults = apply_filters( 'va_cover_defaults', array(
		'images'     => false,
		'object_ids' => false,
		'size'       => 'large',
	) );

	$args  = wp_parse_args( $args, $defaults );
	$image = false;
	$post  = get_post();

	// Heavy lifting. Based on the page/post/taxonomy/author, figure out which image to show, if any.
	if ( ( is_home() && ! in_the_loop() ) || ( ! in_the_loop() && is_singular( 'post' ) ) ) {
		$image = wp_get_attachment_image_src( get_post_thumbnail_id( get_option( 'page_for_posts' ) ), $args[ 'size' ] );
	} else if ( ( ! did_action( 'loop_start' ) && is_archive() && ! is_author() )
		|| ( ! did_action( 'loop_start' ) && is_year() ) // Also covers month and day since year is parent in url.
		|| ( $args[ 'images' ] || $args[ 'object_ids' ] )
	) {
		$thumbnail_id = get_term_meta( get_queried_object_id(), 'thumbnail_id', true );
		if ( 0 < (int) $thumbnail_id ) {
			$image = wp_get_attachment_image_src( $thumbnail_id, $args[ 'size' ] );
		} else {
			// If no custom image was found, grab a random one from the group.
			$image = va_tax_cover_image( $args );
		}
	} else if ( ( ! did_action( 'loop_start' ) && is_archive() && ! is_author() ) || ( $args[ 'images' ] || $args[ 'object_ids' ] ) ) {
		$image = va_tax_cover_image( $args );
	} else if ( ( ! did_action( 'loop_start' ) && is_author() ) ) {
		$author = get_user_by( 'id', get_query_var( 'author' ) );
		$id     = get_user_meta( $author->ID, 'cover_image', true );
		$image  = wp_get_attachment_image_src( $id, $args[ 'size' ] );
	} else if ( is_a( $post, 'WP_Post' ) ) {
		if ( '' != $post->_thumbnail_id ) {
			$image = wp_get_attachment_image_src( get_post_thumbnail_id(), $args[ 'size' ] );
		}
	}

	/**
	 * Filters the background cover photo image url and arguments.
	 *
	 * @since 4.0.0
	 *
	 * @param string $image The url to the image.
	 * @param array  $args  The arguments for the image.
	 * @return string $atts
	 */
	$image = apply_filters( 'va_cover_image', $image, $args );

	if ( ! $image ) {
		$class .= ' no-image';
		return sprintf( 'class="%s"', $class );
	}

	$class .= ' has-image';

	$atts[] = sprintf( 'style="background-image: url(%s);"', $image[0] );
	$atts[] = sprintf( 'class="%s"', $class );

	return implode( ' ', $atts );
}
add_filter( 'va_background_cover', 'va_background_cover', 10, 2 );

/**
 * Checks if we've got a background cover image.
 *
 * @since 4.0.0
 *
 * @param string $image The url to the image.
 * @param array  $args  The arguments for the image.
 * @return string $image
 */
function va_cover_image_check( $image, $args ) {

	// Set a filter so we can check for it later
	// and display an "Add cover image" link.
	if ( ! $image ) {
		//add_filter( 'va_cover_image_none' );
	}

	return $image;
}
add_filter( 'va_cover_image', 'va_cover_image_check', 10, 2 );

/**
 * Display an "Add image" link for admins on pages, tags, categories
 * that don't have a cover image set.
 *
 * @since   4.0.0
 *
 * @return  string $output The url and text link.
 */
function va_cover_image_link() {
	global $wp_the_query;

	// if ( ! has_filter( 'va_cover_image_none' ) || ( ! current_user_can( 'manage_options' ) ) ) {
	// 	return;
	// }

	if ( ! current_user_can( 'manage_options' ) ) {
		return;
	}

	$current_object = $wp_the_query->get_queried_object();

	if ( empty( $current_object ) ) {
		return;
	}

	$url = $text = '';

	if ( ! empty( $current_object->post_type )
		&& ( $post_type_object = get_post_type_object( $current_object->post_type ) )
		&& $edit_post_link = get_edit_post_link( $current_object->ID ) )
	{
		$url   = $edit_post_link;
		//$label = $post_type_object->labels->edit_item;
		$text  = __( 'Edit this page and then click, "Select image" next to the "Cover Image" label.', APP_TD );

	} elseif ( ! empty( $current_object->taxonomy )
		&& ( $tax = get_taxonomy( $current_object->taxonomy ) )
		&& $edit_term_link = get_edit_term_link( $current_object->term_id, $current_object->taxonomy ) )
	{
		$url   = $edit_term_link;
		//$label = $tax->labels->edit_item;
		$text  = __( 'Edit this page and then click, "Set featured image" in the "Featured Image" box.', APP_TD );
	}

	echo '<span class="edit-cover-image-link text-small"><a href="' . esc_url( $url ) . '">' . __( '<i class="fa fa-camera" aria-hidden="true"></i> Edit cover image', APP_TD ) . '</a>' . va_tooltip_icon( $text ) . '</span>';
}
add_action( 'va_home_template_header', 'va_cover_image_link' );
add_action( 'va_blog_template_header', 'va_cover_image_link' );
add_action( 'va_archive_template_header', 'va_cover_image_link' );
add_action( 'va_single_template_header', 'va_cover_image_link' );

/**
 * Add a post class based on certain content being present.
 *
 * @since 4.0.0
 *
 * @param array $classes An array of css post classes.
 */
function va_add_post_class( $classes ) {
	global $post;

	if ( ! has_post_thumbnail() ) {
		$classes[] = 'no-thumbnail';
	}

	if ( '' == get_the_content() ) {
		$classes[] = 'no-content';
	}

	return $classes;
}
add_filter( 'post_class', 'va_add_post_class' );

/**
 * Get a cover image from the same taxonomy (category/post_tag) if one doesn't already exist.
 *
 * @since 4.0.0
 *
 * @param array|object $args
 * @return array $image
 */
function va_tax_cover_image( $args ) {

	$image = false;

	if ( empty( $args[ 'object_ids' ] ) && ( ! isset( $args[ 'images' ] ) || empty( $args[ 'images' ] ) ) ) {

		global $wp_query, $wpdb;

		if ( empty( $wp_query->posts ) ) {
			return $image;
		}

		$args[ 'object_ids' ] = wp_list_pluck( $wp_query->posts, 'ID' );
	}

	if ( ( ! isset( $args[ 'images' ] ) || empty( $args[ 'images' ] ) ) && ( isset( $args[ 'object_ids' ] ) && ! empty( $args[ 'object_ids' ] ) ) ) {

		// No image so query for a new one in the same category/tax.
		if ( ! $image ) {
			$attachments = new WP_Query( array(
				'post_parent__in'        => $args[ 'object_ids' ],
				'post_type'              => 'attachment',
				'post_status'            => 'inherit',
				'posts_per_page'         => 1,
				'orderby'                => 'rand',
				'update_post_term_cache' => false,
				'no_found_rows'          => true,
      ) );

			// Get the first image it finds.
			if ( $attachments->have_posts() ) {
				$image = wp_get_attachment_image_src( $attachments->posts[0]->ID, $args[ 'size' ] );
			}
		}

	} elseif ( isset( $args[ 'images' ] ) && ! empty( $args[ 'images' ] ) ) {
		shuffle( $args[ 'images' ] );
		$image = wp_get_attachment_image_src( current( $args[ 'images' ] ), $args[ 'size' ] );
	}

	/**
	 * Filters the cover image from the same taxonomy.
	 *
	 * @since 4.0.0
	 *
	 * @param string $image The url to the image.
	 * @param array  $args  The arguments for the image.
	 */
	return apply_filters( 'va_tax_cover_image', $image, $args );
}

/**
 * Retrieve protected post password form content to set the button class.
 *
 * @since 4.0.0
 *
 * @param int|WP_Post $post Optional. Post ID or WP_Post object. Default is global $post.
 * @return string HTML content for password form for password protected post.
 */
function va_the_password_form( $post = 0 ) {

	$post = get_post( $post );
	$label = 'pwbox-' . ( empty( $post->ID ) ? rand() : $post->ID );
	$output = '<form action="' . esc_url( site_url( 'wp-login.php?action=postpass', 'login_post' ) ) . '" class="post-password-form" method="post">
	<p>' . __( 'This content is password protected. To view it, enter your password below.', APP_TD ) . '</p>
	<p><label for="' . $label . '">' . __( 'Password', APP_TD ) . ' <input name="post_password" id="' . $label . '" type="password"></label> <input type="submit" name="Submit" value="' . esc_attr__( 'Submit', APP_TD ) . '" class="button small" /></p></form>
	';

	return $output;
}
add_filter( 'the_password_form', 'va_the_password_form' );

/**
 * Generates a unique ID for listings.
 *
 * @return string
 */
function va_generate_id() {

	$id = uniqid( rand( 10, 1000 ), false );

	if ( va_get_listing_by_ref( $id ) ) {
		return va_generate_id();
	}

	return $id;
}

/**
 * Generates a clickable phone number for a tel: href usage.
 *
 * @since 4.0.0
 *
 * @param string $phone The phone number text string.
 * @return string
 */
function va_phone_url( $phone ) {
	return preg_replace( '/[^0-9,.+]/', '', $phone );
}

/**
 * Output the map html placeholder content.
 *
 * @since 4.0.0
 */
function va_map_canvas() {
	ob_start();
?>
	<div id="mapCanvas"></div>
	<div id="map-loading">
		<h4><?php _e( 'Loading Map', APP_TD );?></h4>
		<div class="spinner map_loader" id="listing_loader_maps">
			<div class="rect1"></div>
			<div class="rect2"></div>
			<div class="rect3"></div>
			<div class="rect4"></div>
			<div class="rect5"></div>
		</div>
	</div>
	<div id="map-zoom-in"><i class="fa fa-plus"></i></div>
	<div id="map-zoom-out"><i class="fa fa-minus"></i></div>
<?php
	$map_canvas = ob_get_clean();

	/**
	 * Filters the Google maps canvas html which javascript depends on.
	 *
	 * @since 4.0.0
	 *
	 * @param string $map_canvas The map canvas html.
	 */
	echo apply_filters( 'va_map_canvas', $map_canvas );
}

/**
 * Generate icon tooltips via Foundation Framework's tooltip js.
 *
 * @since 4.0.0
 *
 * @param string $text     The text to display in the tooltip.
 * @param string $position (Optional) The placement of the tooltip (e.g. top, left, right).
 * @param string $id       (Optional) The span ID in case you want to control differently.
 * @param string $icon     (Optional) The Font Awesome icon you want to use. Defaults to fa-question-circle.
 * @return string
 */
function va_tooltip_icon( $text = '', $position = 'top', $id = '', $icon = 'fa-question-circle' ) {

	$output = '<span data-tooltip aria-haspopup="true" id="' . esc_attr( $id ) . '" class="va-tooltip has-tip ' . esc_attr( $position ) . ' " data-disable-hover="false" tabindex="2" title="' . esc_attr( $text ) . '"><i class="fa ' . esc_attr( $icon ) . '" aria-hidden="true"></i></span>';

	/**
	 * Filters the icon tooltip.
	 *
	 * @since 4.0.0
	 *
	 * @param string $text     The text to display in the tooltip.
	 * @param string $position (Optional) The placement of the tooltip (e.g. top, left, right).
	 * @param string $id       (Optional) The span ID in case you want to control differently.
	 * @param string $icon     (Optional) The Font Awesome icon you want to use. Defaults to fa-question-circle.
	 */
	return apply_filters( 'va_tooltip_icon', $output, $text, $position, $id, $icon );
}
