<?php
/**
 * Contact listing owner form and ajax processing.
 *
 * @package Vantage
 * @since 4.0.0
 */

/**
 * Display the contact listing owner button and modal window.
 *
 * @since 4.0.0
 */
function va_contact_listing_owner() {
	global $post;

	if ( get_post_meta( $post->ID, 'listing_claimable', true ) || 'publish' !== get_post_status( $post->ID ) ) {
		return;
	}

	$display_name = '';
	$user_email   = '';

	// Load validation scripts for modal form.
	wp_enqueue_script( 'validate' );
	wp_enqueue_script( 'validate-lang' );

	// Autofill the fields if user is logged in.
	if ( $user = wp_get_current_user() ) {
		$display_name = $user->display_name;
		$user_email   = $user->user_email;
	}

	ob_start();
?>

	<p><a data-open="contactOwnerModal" class="button hollow expanded" id="contact-owner-link"><?php _e( 'Contact Owner', APP_TD ); ?></a></p>

	<!-- Modal window -->
	<div class="reveal" id="contactOwnerModal" data-reveal>

	  <p class="lead"><?php _e( 'Contact Owner', APP_TD ); ?></p>

		<p><?php _e( 'Complete the form below to send a message to this owner.', APP_TD ); ?></p>

		<div id="va-contact-form-response"></div>

		<i class="fa fa-spinner fa-spin fa-3x fa-fw"></i>

		<div id="va-contact-form-wrap">

			<form id="va-contact-form" class="va-contact-form" action="post">

				<?php
				/**
				 * Fires before the fields in the contact owner form.
				 *
				 * @since 4.0.0
				 */
				do_action( 'va_contact_owner_form_before_fields' );
				?>

				<label><?php _e( 'Name', APP_TD ); ?>
					<input type="text" name="contact_name" id="contact_name" class="required" value="<?php echo esc_attr( $display_name ); ?>">
				</label>

				<label><?php _e( 'Email', APP_TD ); ?>
					<input type="email" name="contact_email" id="contact_email" class="required email" value="<?php echo esc_attr( $user_email ); ?>">
				</label>

				<label><?php _e( 'Message', APP_TD ); ?>
					<textarea name="contact_message" id="contact_message" rows="5" class="required"></textarea>
				</label>

				<!-- We hide this with CSS (spam trap), that's why it has an ID. -->
				<label id="contact_catcher">Username
					<input type="text" name="contact_username" id="contact_username" autocomplete="off">
				</label>

				<?php
				/**
				 * Fires after the fields in the contact owner form.
				 *
				 * @since 4.0.0
				 */
				do_action( 'va_contact_owner_form_after_fields' );
				?>

				<input type="submit" id="va-contact-submit" class="button expanded" value="<?php esc_attr_e( __( 'Send Inquiry', APP_TD ) ); ?>">

				<?php wp_nonce_field( 'vantage_contact_owner_nonce' ); ?>
				<input type="hidden" name="post_ID" id="post_ID" value="<?php echo $post->ID; ?>">

			</form>

		</div>

		<button class="close-button" data-close aria-label="<?php echo esc_attr_e( __( 'Close modal window', APP_TD ) ); ?>" type="button">
			<span aria-hidden="true">&times;</span>
		</button>

	</div>
	<!-- Modal window end -->

<?php
	return ob_get_clean();
}


/**
 * Process the ajax contact listing owner form submit.
 *
 * @since 4.0.0
 */
function va_contact_listing_owner_process() {

	// Make sure it's a legit request.
	check_ajax_referer( 'vantage_contact_owner_nonce', 'nonce' );

	// We need certain params to proceed.
	if ( ! isset( $_POST[ 'id' ] ) ) {
		wp_send_json_error( __( 'Post ID is required.', APP_TD ) );
	}

	// Make sure it's a real post.
	$post = get_post( $_POST[ 'id' ] );

	// Make sure post is valid.
	if ( ! $post ) {
		wp_send_json_error( __( 'Post not found.', APP_TD ) );
	}

	// $error = new WP_Error( '-2', 'The post ID was not set' );
	// wp_send_json_error( $error );

	// Clean the fields.
	if ( isset( $_POST['honeypot'] ) ) {
		$honeypot = trim( $_POST['honeypot'] );
	}

	if ( isset( $_POST['name'] ) && is_string( $_POST['name'] ) ) {
		$contact_name = trim( strip_tags( $_POST['name'] ) );
	}

	if ( isset( $_POST['email'] ) && is_string( $_POST['email'] ) ) {
		$contact_email = trim( $_POST['email'] );
	}

	if ( isset( $_POST['message'] ) && is_string( $_POST['message'] ) ) {
		$contact_message = trim( $_POST['message'] );
	}

	// Validate the fields.
	if ( $honeypot ) {
		wp_send_json_error( __( "You triggered our spam trap. Disable your browsers' autocomplete feature and try again.", APP_TD ) );
	}

	if ( 6 > strlen( $contact_email ) || '' == $contact_name ) {
		wp_send_json_error( __( 'Name and email are required fields.', APP_TD ) );
	}

	if ( '' == $contact_message ) {
		wp_send_json_error( __( 'Please enter a message.', APP_TD ) );
	}

	// Passed validation. Let's generate the email.
	ob_start();

	appthemes_load_template( 'email-contact.php', $form_fields );

	$content = ob_get_clean();

	// Get the listing owner object.
	$recipient = get_user_by( 'id', $post->post_author );

	// Send this email if no template is found.
	if ( empty( $content ) ) {
		$content .= html( 'p', sprintf( __( 'Hi %s,', APP_TD ), $recipient->display_name ) );
		$content .= html( 'p', sprintf( __( 'Someone has contacted you regarding your "%s" listing.', APP_TD ), $post->post_title ) );
		$content .= sprintf( __( 'Name: %s', APP_TD ), $contact_name . '<br>' );
		$content .= sprintf( __( 'Email: %s', APP_TD ), $contact_email . '<br>' );
		$content .= html( 'p', sprintf( __( 'Message: %s', APP_TD ), '<br>' . $contact_message . '<br><br>----' ) );
		$content .= html( 'p', sprintf( __( 'Source: <a href="%1$s" target="_blank">%1$s</a>', APP_TD ), get_permalink( $post->ID ) ) );
	}

	$subject = sprintf( __( 'New inquiry on "%s"', APP_TD ), $post->post_title );

	APP_Mail_From::apply_once( array( 'email' => $contact_email, 'name' => $contact_name, 'reply' => true ) );

	$args = array(
		'to'          => $recipient->user_email,
		'subject'     => $subject,
		'message'     => $content,
		'listing'     => $post,
		'contact_name' => $contact_name,
		'contact_email' => $contact_email,
		'contact_message' => $contact_message,
	);

	// Send the email.
	va_appthemes_send_email( 'va_email_listing_owner', $args );

	/**
	 * Fires after the contact listing owner email was sent.
	 *
	 * @since 4.0.0
	 *
	 * @param array              $args The email field values.
	 * @param WP_Post|array|null $post The post object.
	 */
	do_action( 'va_contact_listing_owner_success', $args, $post );

	// Return the response.
	wp_send_json_success( __( 'Your message was successfully sent!', APP_TD ) );
}
add_action( 'wp_ajax_vantage_contact_owner', 'va_contact_listing_owner_process' );
add_action( 'wp_ajax_nopriv_vantage_contact_owner', 'va_contact_listing_owner_process' );
