<?php
/**
 * Business Listing Form submodule
 *
 * @package Vanatge\Listing
 * @author  AppThemes
 * @since   4.0.0
 */

/**
 * Form processing class
 */
class VA_Business_Listing_Form extends APP_Listing_Form {

	/**
	 * Contact metabox object.
	 *
	 * @var VA_Business_Listing_Contact_Metabox
	 */
	public $contact_box;

	/**
	 * Constructs listing form object
	 *
	 * @param APP_Listing $listing Listing object to assign process with.
	 */
	public function __construct( APP_Listing $listing ) {

		$this->listing = $listing;

		parent::__construct( $listing );

		add_action( 'appthemes_handle_media_field', array( $this, 'set_post_thumbnail' ), 10, 2 );
	}

	/**
	 * Activates submodules
	 */
	protected function activate_modules() {

		if ( is_admin() ) {
			if ( ! $this->fields_box ) {
				$this->fields_box = new VA_Business_Listing_Fields_Metabox( $this->listing, __( 'Other Fields', APP_TD ) );
			}

			if ( ! $this->contact_box ) {
				$this->contact_box = new VA_Business_Listing_Contact_Metabox( $this->listing, __( 'Contact Information', APP_TD ) );
			}
		}

		parent::activate_modules();
	}

	/**
	 * Retrieves core fields
	 *
	 * @return array An array of the fields parameters
	 */
	public function get_core_fields() {

		$social_fields = array();

		foreach ( va_get_allowed_listing_networks() as $social_network ) {
			$social_fields[] = array(
				'type'  => 'input_text',
				'id'    => $social_network,
				'props' => array(
					'required' => 0,
					'label'    => va_get_social_network_title( $social_network ),
					'tip'      => va_get_social_network_tip( $social_network ),
				),
			);
		}

		$fields = array(
			array(
				'type'  => 'input_text',
				'id'    => 'post_title',
				'props' => array(
					'required' => 1,
					'label'    => __( 'Title', APP_TD ),
				),
			),
			array(
				'type'  => 'input_text',
				'id'    => 'address',
				'props' => array(
					'required' => 1,
					'label'    => __( 'Location', APP_TD ),
				),
			),
			array(
				'type'  => 'tax_input',
				'id'    => 'tax_input[' . VA_LISTING_CATEGORY . ']',
				'props' => array(
					'required'   => 1,
					'label'      => __( 'Categories', APP_TD ),
					'tax'        => VA_LISTING_CATEGORY,
					'field_type' => 'checkbox',
				),
			),
			array(
				'type'  => 'input_text',
				'id'    => 'phone',
				'props' => array(
					'required' => 0,
					'label'    => __( 'Phone', APP_TD ),
				),
			),
			array(
				'type'  => 'url',
				'id'    => 'website',
				'props' => array(
					'required' => 0,
					'label'    => __( 'Website', APP_TD ),
				),
			),
			array(
				'type'  => 'email',
				'id'    => 'email',
				'props' => array(
					'required' => 0,
					'label'    => __( 'Email', APP_TD ),
				),
			),
		);

		$detail_fields = array(
			array(
				'type'  => 'file',
				'id'    => '_app_media',
				'props' => array(
					'required'    => 1,
					'label'       => __( 'Listing Media', APP_TD ),
					'extensions'  => 'Image',
					'file_limit'  => 5,
					'embed_limit' => 0,
					'file_size'   => '',
				),
			),
			array(
				'type'  => 'textarea',
				'id'    => 'post_content',
				'props' => array(
					'required'    => 1,
					'label'       => __( 'Description', APP_TD ),
					'editor_type' => 'tmce',
				),
			),
			array(
				'type'  => 'tax_input',
				'id'    => 'tax_input[' . VA_LISTING_TAG . ']',
				'props' => array(
					'required'   => 0,
					'label'      => __( 'Tags', APP_TD ),
					'tip'        => __( 'Separate tags with commas', APP_TD ),
					'tax'        => VA_LISTING_TAG,
					'field_type' => 'text',
				),
			),
		);

		$fields = array_merge( $fields, $social_fields, $detail_fields );

		return apply_filters( 'appthemes_core_fields', $fields, $this->listing->get_type() );
	}

	public function apply_atts( $field, $item_id ) {

		$field = parent::apply_atts( $field, $item_id );

		// Check for any files uploaded before 4.0 and upgrade it to the new media manager.
		if ( 'file' === $field['type'] ) {

			if ( ! in_array( $field['name'], (array) get_post_custom_keys( $item_id ) ) ) {

				$attachments = array();

				if ( '_app_media' === $field['name'] ) {
					$attachments = va_get_post_attachments( $item_id, -1, VA_ATTACHMENT_GALLERY, 'ids' );
				} else {
					$_attachments = va_get_post_attachments( $item_id, -1, VA_ATTACHMENT_FILE, 'ids' );

					foreach ( $_attachments as $attach_id ) {
						if ( get_post_meta( $attach_id, '_va_field_' . $field['name'], true ) ) {
							$attachments[] = $attach_id;
						}
					}
				}

				update_post_meta( $item_id, $field['name'], $attachments );

				foreach ( $attachments as $attach_id ) {
					update_post_meta( $attach_id, '_app_attachment_type', 'file' );
				}
			}
		}

		if ( isset( $field['tip'] ) ) {
			$name = scbForms::get_name( $field['name'] );
			$field['extra']['aria-describedby'] = "{$name}HelpText";
		}

		$field['desc_pos'] = 'after';

		return $field;
	}

	/**
	 * Wraps form field token and adds label and description.
	 *
	 * @param array $token Field input token to be replaced with generated html.
	 * @param array $field Field parameters.
	 *
	 * @return string Wrapped field html.
	 */
	public function field_wrap( $token, $field ) {
		$name  = scbForms::get_name( $field['name'] );
		$label = html( 'label', array( 'for' => $name ), $field['title'] );
		$type  = $field['props']['type'];
		$tip   = '';

		if ( isset( $field['tip'] ) ) {
			$tip = html( 'p class="help-text" id="' . esc_attr( $name ) . 'HelpText"', $field['tip'] );
		}

		$html = html( 'div class="form-field ' . $type . '"', $label . $token . $tip );

		return $html;
	}

	public function set_post_thumbnail( $item_id, $field ) {
		global $post;

		if ( is_admin() ) {
			return;
		}

		if ( '_app_media' !== $field ) {
			return;
		}

		if ( $item_id ) {
			 $item = get_post( $item_id );
		}

		// Set up featured images only for services and blog posts
		if ( !  $item || $this->listing->get_type() !== $item->post_type ) {
			return;
		}

		$attachment_ids = appthemes_get_post_attachments( $item_id, $filters = array(
			'mime_types' => 'image',
			'file_limit' => 1,
		) );

		if ( ! empty( $attachment_ids ) ) {
			set_post_thumbnail(  $item->ID, $attachment_ids[0] );
		}
	}

}
