<?php
/**
 * Listing geocode address class.
 *
 * This module is responsible for listing address processing.
 *
 * @package Vanatge\Listing
 * @author  AppThemes
 * @since   4.0.0
 */

/**
 * Listing Geocode Address class.
 */
class VA_Business_Listing_Geocode {

	/**
	 * Current Listing module object.
	 * @var APP_Listing
	 */
	protected $listing;

	/**
	 * The listing address form field name.
	 * @var string
	 */
	protected $name;

	/**
	 * Constructs listing geocode address object.
	 *
	 * @param APP_Listing $listing  Listing object to assign process with.
	 * @param string      $name     Address form field name.
	 */
	public function __construct( APP_Listing $listing, $name = 'address' ) {

		$this->listing = $listing;
		$this->name    = $name;

		add_filter( 'appthemes_form_field', array( $this, 'form_field' ), 10, 2 );

	}

	/**
	 * Adds extra parameters to listing form field.
	 *
	 * @param array $field   Form field.
	 * @param int   $item_id Listing item ID.
	 *
	 * @return array Modified Form field.
	 */
	public function form_field( $field, $item_id ) {

		if ( $this->get_name() !== $field['name'] ) {
			return $field;
		}

		if ( $this->listing->get_type() !== get_post_type( $item_id ) ) {
			return $field;
		}

		$field['listing_id'] = $item_id;

		// Set custom validator.
		$field['sanitizers'][] = array( $this, 'validate' );

		// Set CSS class to use with jQuery.appAddressAutocomplete plugin.
		if ( ! isset( $field['extra'] ) || ! isset( $field['extra']['class'] ) ) {
			$field['extra']['class'] = 'app-address-field';
		} else {
			$field['extra']['class'] .= ' app-address-field';
		}

		$field['extra']['class'] = trim( $field['extra']['class'] );

		return $field;
	}

	/**
	 * Validates field.
	 *
	 * Checks if address field was changed or never geocoded and try to update
	 * geocoder data before field save.
	 *
	 * @todo Need to get API server response messages on error and put into
	 * given error object. So field won't be updated on server API errors.
	 *
	 * @param mixed          $value  Posted field value.
	 * @param scbCustomField $inst   Field object.
	 * @param WP_Error       $errors Errors object.
	 *
	 * @return mixed Validated value.
	 */
	public function validate( $value, $inst, $errors ) {

		$prev_value = get_post_meta( $inst->listing_id, $inst->name, true );
		$has_coords = APP_Google_Geocoding::has_geo_data( $inst->listing_id );
		$data       = array();

		if ( $has_coords && $value === $prev_value ) {
			return $value;
		}

		$data_key = $inst->name . '_geo_data';

		if ( ! empty( $_POST[ $data_key ] ) ) {
			parse_str( $_POST[ $data_key ], $data );
			$data = wp_unslash( $data );
			$data['raw_address'] = $value;
			$data['api_response'] = 'OK: ' . __( 'Successfully geocoded address.', APP_TD );

			$data = APP_Google_Geocoding::build_geo_data_array( $data );
		}

		// @todo: It needs to catch API errors here.
		APP_Google_Geocoding::update_geo_data( $inst->listing_id, $value, $data );

		return $value;
	}

	/**
	 * Retrieves address field name.
	 * @return string
	 */
	public function get_name() {
		return $this->name;
	}
}
