<?php
/**
 * Google geocoding class.
 *
 * @todo Eventually move this into framework. Move $va_options to framework.
 *
 * @package Framework
 * @since 4.0.0
 */

/**
 * Google geocoding main class.
 *
 * @since 4.0.0
 */
class APP_Google_Geocoding {

	/**
	 * The prefix for a stored geo meta value.
	 *
	 * @since 4.0.0
	 *
	 * @var string
	 */
	private static $prefix = 'geo_';

	/**
	 * The array of reserved geocoding fields. Where keys are list of internal
	 * names and values are the names of appropriate meta fields.
	 *
	 * @var type
	 */
	protected static $fields;

	/**
	 * Init.
	 */
	public static function init() {

		$fields = array(
			'lat'               => 'geo_lat',
			'lng'               => 'geo_lng',
			'raw_address'       => 'geo_raw_address',
			'formatted_address' => 'geo_formatted_address',
			'place_id'          => 'geo_place_id',
			'street_number'     => 'geo_street_number',
			'street'            => 'geo_street',
			'city'              => 'geo_city',
			'state_short'       => 'geo_state_short',
			'state_long'        => 'geo_state_long',
			'postal_code'       => 'geo_postal_code',
			'country_short'     => 'geo_country_short',
			'country_long'      => 'geo_country_long',
		);

		/**
		 * Filters the default Google geocode fields.
		 *
		 * @since 4.0.0
		 *
		 * @param array $fields The default fields and values.
		 */
		self::$fields = apply_filters( 'appthemes_google_geo_fields', $fields );
	}

	/**
	 * A whitelist of all the geo fields we use.
	 *
	 * @since  4.0.0
	 *
	 * @return array The geo fields. Where keys are list of internal names and
	 *               values are the names of appropriate meta fields.
	 */
	public static function geo_fields() {
		return self::$fields;
	}

	/**
	 * Returns all the geo meta fields
	 *
	 * @since 4.0.0
	 *
	 * @param int $post_id The post ID to check against.
	 */
	public static function get_post_meta( $post_id ) {

		foreach ( self::geo_fields() as $key => $value ) {
			$fields[ $key ] = get_post_meta( $post_id, $value, true );
		}

		return $fields;
	}

	/**
	 * Insert geo data on new listings.
	 *
	 * @since 4.0.0
	 *
	 * @param int    $post_id The post ID to save values to.
	 * @param string $address The text field of the new address.
	 * @param array  $data    Data to be saved. (Optional). If empty, new API
	 *                         request will be performed.
	 */
	public static function insert_geo_data( $post_id, $address, $data = array() ) {

		// Need an address to proceed.
		if ( false == trim( $address ) ) {
			return;
		}

		if ( empty( $data ) ) {
			$data = self::get_geo_data( $address );
		}

		self::save_geo_data( $post_id, $data );
	}

	/**
	 * Update the existing geocoded data fields.
	 *
	 * @since 4.0.0
	 *
	 * @param int    $post_id The post ID to update values on.
	 * @param string $address The text field of the new address.
	 * @param array  $data    Data to be saved. (Optional). If empty, new API
	 *                         request will be performed.
	 */
	public static function update_geo_data( $post_id, $address, $data = array() ) {

		// Need an address to proceed.
		if ( false == trim( $address ) ) {
			self::delete_geo_data( $post_id );
			return;
		}

		if ( empty( $data ) ) {
			$data = self::get_geo_data( $address );
		}

		self::delete_geo_data( $post_id );
		self::save_geo_data( $post_id, $data );
	}

	/**
	 * Save the geocoded data fields to the database.
	 *
	 * @since 4.0.0
	 *
	 * @param int   $post_id The post ID to save values to.
	 * @param array $values  The array of geo data to process.
	 */
	protected static function save_geo_data( $post_id, $values ) {

		if ( ! is_wp_error( $values ) && $values ) {

			$whitelist = self::geo_fields();

			foreach ( $values as $key => $value ) {
				if ( $value ) {
					$meta_key = ( isset( $whitelist[ $key ] ) ) ? $whitelist[ $key ] : self::$prefix . $key;
					update_post_meta( $post_id, $meta_key, $value );
				}
			}
		} else {
			update_post_meta( $post_id, self::$prefix . 'error_message', $values->get_error_message() );
		}
	}

	/**
	 * Delete the geo coded data fields from the database.
	 *
	 * @since 4.0.0
	 *
	 * @param int $post_id The post ID to save values to.
	 */
	public static function delete_geo_data( $post_id ) {
		foreach ( self::geo_fields() as $key => $value ) {
			delete_post_meta( $post_id, $value );
		}
	}

	/**
	 * Check to see if address has already been geocoded.
	 *
	 * @since 4.0.0
	 *
	 * @param int $post_id The post ID to check against.
	 * @return string The geo data last updated date.
	 */
	public static function has_geo_data( $post_id ) {
		$fields = self::geo_fields();
		return get_post_meta( $post_id, $fields['lat'], true );
	}

	/**
	 * Builds an array of geocode data.
	 *
	 * @param array|object $data Raw returned geocoded data.
	 *
	 * @return array Standardized geo data.
	 */
	public static function build_geo_data_array( $data ) {

		$output   = array();
		$defaults = array_keys( self::geo_fields() );
		$defaults = array_merge( $defaults, array( 'raw_address', 'api_response' ) );
		$data     = (array) $data;

		// Fill known whitelisted values.
		foreach ( $defaults as $key ) {
			$output[ $key ] = ( isset( $data[ $key ] ) ) ? $data[ $key ] : '';
		}

		// Make sure the address components are there.
		if ( ! empty( $data['address_components'] ) ) {

			foreach ( $data['address_components'] as $key ) {

				$key = (array) $key;

				switch ( $key['types'][0] ) {

					case 'street_number' :
						$output['street_number'] = $key['long_name'];
					break;
					case 'route' :
						$output['street'] = $key['long_name'];
					break;
					case 'sublocality_level_1' :
					case 'locality' :
					case 'postal_town' :
						$output['city'] = $key['long_name'];
					break;
					case 'administrative_area_level_1' :
					case 'administrative_area_level_2' :
						$output['state_short'] = $key['short_name'];
						$output['state_long']  = $key['long_name'];
					break;
					case 'postal_code' :
						$output['postal_code'] = $key['long_name'];
					break;
					case 'country' :
						$output['country_short'] = $key['short_name'];
						$output['country_long']  = $key['long_name'];
					break;
				}
			}
		}

		/**
		 * Filters the default Google Geocoding params.
		 *
		 * @since 4.0.0
		 *
		 * @param array  $output   The processed geo param values.
		 * @param string $data     The original returned geo param values.
		 */
		$output = apply_filters( 'appthemes_google_geo_data_results', $output, $data );

		return $output;
	}

	/**
	 * Fetch results from Google's Geocoding API.
	 *
	 * @since  4.0.0
	 *
	 * @param  string $address The address provided from an input field.

	 * @return array The returned values from Google or error.
	 */
	public static function get_geo_data( $address ) {
		global $va_options;

		$output = array();

		// Store for later use.
		$raw_address = $address;

		$address = self::clean_address( $address );

		// Need an address to proceed.
		if ( empty( $address ) ) {
			return false;
		}

		try {

			// Encode appropriately before using add_query_arg().
			$params = array(
				'address' => rawurlencode( $address ),
				'region'  => strtolower( $va_options->google_maps_region ),
				'key'     => $va_options->google_maps_api_key,
			);

			/**
			 * Filters the default Google Geocoding params.
			 *
			 * @since 4.0.0
			 *
			 * @param array $params The default geo param values.
			 */
			$params = apply_filters( 'appthemes_google_geo_default_params', $params );

			$args = array(
				'timeout'     => 5,
				'httpversion' => '1.1',
				'user-agent'  => 'WordPress/AppThemes-' . get_option( 'current_theme' ) . '; ' . get_bloginfo( 'url' ),
			);

			// Query the Google API.
			$response = wp_remote_get( add_query_arg( $params, 'https://maps.googleapis.com/maps/api/geocode/json' ), $args );

			// Get the response.
			$results = json_decode( wp_remote_retrieve_body( $response ) );

			// Set the default response vars.
			$status    = 'ERROR';
			$msg       = '';

			// Status returned means we connected to Google.
			if ( $results->status ) {

				$status = $results->status;

				// Go through all possible return codes from Google.
				// https://developers.google.com/maps/documentation/geocoding/intro#GeocodingResponses
				switch ( $status ) {

					case 'OK' :
						$msg  = __( 'Successfully geocoded address.', APP_TD );
					break;
					case 'ZERO_RESULTS' :
						$msg = __( 'No results found for address.', APP_TD );
					break;
					case 'OVER_QUERY_LIMIT' :
					case 'REQUEST_DENIED' :
					case 'INVALID_REQUEST' :
					case 'UNKNOWN_ERROR' :
						// Error msg provided by API.
						$msg = $results->error_message;
					break;
					default :
						$msg = __( 'General geocoding error. Try again later.', APP_TD );
					break;
				}
			} else {
				$msg = __( 'No status message returned.', APP_TD );
			}
		} catch ( Exception $e ) {
			return new WP_Error( 'error', $e->getMessage() );
		}

		if ( $status != 'OK' ) {
			return new WP_Error( 'error', $msg );
		}

		// Set a shortcut variable.
		$r = $results->results[0];

		$r->lat          = $r->geometry->location->lat;
		$r->lng          = $r->geometry->location->lng;
		$r->raw_address  = $raw_address;
		$r->api_response = $status . ': ' . $msg;

		$output = self::build_geo_data_array( $r );

		// Google seems to handle this already but extra
		// check to make sure stored values are clean.
		$output = array_map( 'sanitize_text_field', $output );

		return $output;
	}

	/**
	 * Remove some reserved characters so it's url-safe.
	 * https://developers.google.com/maps/web-services/overview#BuildingURLs
	 *
	 * @since 4.0.0
	 *
	 * @param  string $address Raw address from user input
	 * @return string The clean address string
	 */
	public static function clean_address( $output ) {
		// Leave out some chars since they could be used and encoded * ' @ , # /
		return str_replace( array( '!', '(', ')', ';', ':', '=', '&', '+', '$', '?', '%', '[', ']' ), '', $output );
	}

}
