<?php
/**
 * Google maps class.
 *
 * @todo Eventually move this into framework. Move $va_options to framework. Use different hooks?
 *
 * @package Vantage
 * @since 4.0.0
 */

/**
 * Google maps main class.
 *
 * @since   4.0.0
 */
class APP_Google_Maps {

	/**
	 * The page we're on for options.
	 *
	 * @since 4.0.0
	 */
	private static $page;

	/**
	 * Constructor
	 */
	public function __construct() {

		add_action( 'wp_enqueue_scripts',             array( $this, 'enqueue_scripts' ) );
		// add_filter( 'script_loader_tag',           array( $this, 'add_defer_attribute' ), 10, 2);
		add_action( 'va_listings_map_canvas',         array( $this, 'map_canvas' ) );
		add_action( 'tabs_vantage_page_app-settings', array( $this, 'admin_fields' ) );
		add_action( 'admin_notices',                  array( $this, 'admin_notices' ) );

		// add_filter( 'posts_clauses',               array( $this, 'posts_clauses' ) );
	}

	/**
	 * Load up the map scripts.
	 *
	 * @since  4.0.0
	 *
	 * @return void
	 */
	public function enqueue_scripts() {
		global $va_options;

		wp_enqueue_script( 'at-google-maps-api', $this->map_url(), array(), null, false );
		wp_localize_script( 'at-google-maps-api', 'appthemes_google_map_localize_script', $this->map_settings() );

		// Handle must match main theme stylesheet handle to work.
		wp_add_inline_style( 'theme-styles', $this->inline_css() );
	}

	/**
	 * Add async and defer tags to the maps js script includes.
	 *
	 * "Not defined" error unless we include a callback in the url.
	 *
	 * @since 4.0.0
	 *
	 * @param string $tag    The `<script>` tag for the enqueued script.
	 * @param string $handle The script's registered handle.
	 * @param string $src    The script's source URL.
	 */
	function add_defer_attribute( $tag, $handle, $src ) {

		if ( $handle !== 'at-google-maps-api' ) {
			return $tag;
		}

		return str_replace( ' src', ' async defer src', $tag );
	}

	/**
	 * Default Google api url for the map.
	 *
	 * @since  4.0.0
	 *
	 * @return array
	 */
	public function map_url() {
		global $va_options;

		$defaults = array(
			'libraries' => array( 'geometry', 'places' ),
			'region'    => 'US',
			'language'  => 'en',
			'unit'      => 'mi',
			'key'       => '',
			'callback'  => '',
		);

		/**
		 * Filters the default Google Maps args.
		 *
		 * @since 4.0.0
		 *
		 * @param array $defaults The default map values.
		 */
		$defaults = apply_filters( 'appthemes_google_map_default_args', $defaults );

		$defaults['libraries'] = implode( ',', $defaults['libraries'] );

		$args = array(
			'region'   => strtolower( $va_options->google_maps_region ),
			'language' => strtolower( $va_options->google_maps_language ),
			'key'      => $va_options->google_maps_api_key,
		);

		$args = wp_parse_args( $args, $defaults );
		$url  = add_query_arg( $args, '//maps.googleapis.com/maps/api/js' );

		/**
		 * Filters the Google map url.
		 *
		 * @since 4.0.0
		 *
		 * @param string $url The url for Google maps.
		 * @param array $args The args array for Google maps.
		 */
		return apply_filters( 'appthemes_google_map_url', $url, $args );
	}

	/**
	 * Default settings for the map.
	 *
	 * These can be overridden via filters. Then call them from your js script.
	 * e.g. styles: appthemes_google_map_localize_script.mapOptions.styles
	 *
	 * @since  4.0.0
	 *
	 * @return array
	 */
	public function map_settings() {
		global $va_options;

		$flag = ( $va_options->google_maps_api_key ) ? true : false;

		$settings = array(
			'imgPath'    => get_template_directory_uri() . '/assets/images/',
			'no_geocode' => __( "Autocomplete's returned place contains no geometry", APP_TD ),
			'hasApiKey'  => $flag,
			'mapOptions' => array(
				'backgroundColor' => '#f7f7f7',
				'styles'          => $this->map_styles(),
			),
		);

		/**
		 * Filters the Google map settings array variables.
		 *
		 * @since 4.0.0
		 *
		 * @param array $settings The settings array for Google maps.
		 */
		return apply_filters( 'appthemes_google_map_settings', $settings );
	}

	/**
	 * Custom styles for the map.
	 *
	 * Add more color schemes from https://snazzymaps.com
	 *
	 * @since  4.0.0
	 *
	 * @return array
	 */
	public function map_styles() {
		global $va_options;

		$styles   = '';

		// Load the default color scheme if one hasn't been selected.
		$filename = ( ! empty( $va_options->google_color_scheme ) ) ? $va_options->google_color_scheme : 'standard';

		// Grab the style json file from the schemes folder.
		$scheme = trailingslashit( dirname( __FILE__ ) ) . trailingslashit( 'schemes' ) . $filename . '.json';

		/**
		 * Filters the Google map default style file.
		 *
		 * @since 4.0.0
		 *
		 * @param array $scheme The path to the default style file.
		 */
		$scheme = apply_filters( 'appthemes_google_map_styles_file', $scheme );

		if ( file_exists( $scheme ) ) {
			$data = file_get_contents( $scheme );
		}

		// Convert the json file to a php array.
		if ( ! is_null( $data ) && $data ) {
			$styles = json_decode( $data, true );
		}

		/**
		 * Filters the Google map style array variables.
		 *
		 * Tutorial: https://developers.google.com/maps/documentation/javascript/styling#creating_a_styledmaptype
		 *
		 * @since 4.0.0
		 *
		 * @param array $styles The style array for Google maps.
		 */
		return apply_filters( 'appthemes_google_map_styles', $styles );
	}

	/**
	 * Generate the listings map canvas for Google maps.
	 *
	 * @since 4.0.0
	 * @return string
	 */
	public function map_canvas() {
		ob_start();
	?>
	<div id="listings-map-wrap">
		<div id="listings-map" class="listings-map content-wrap"></div>
		<div id="map-loading">
			<h4><?php _e( 'Loading Map', APP_TD );?></h4>
			<div class="spinner map_loader" id="listing_loader_maps">
				<div class="rect1"></div>
				<div class="rect2"></div>
				<div class="rect3"></div>
				<div class="rect4"></div>
				<div class="rect5"></div>
			</div>
		</div>
		<div id="map-no-results">
			<h4><?php _e( 'No Results Found', APP_TD );?></h4>
		</div>
	</div>
	<?php
		echo ob_get_clean();
	}

	/**
	 * Returns the post ids found in the radius search
	 * so we can limit results using `posts__in`.
	 *
	 * @see VA_Listing_Search_View::parse_query()
	 *
	 * @since 4.0.0
	 *
	 * @return array
	 */
	static function posts_clauses() {
		global $wpdb, $wp_query, $va_options;

		// No geo params so we can't perform the radius search
		if ( empty( $_GET['lat'] ) && empty( $_GET['lng'] ) ) {
			return;
		}

		// Can't use get_query_var() here.
		$lat = (float) $_GET['lat'];
		$lng = (float) $_GET['lng'];

		// If distance is submitted, limit results within that radius.
		$radius = isset( $_GET['radius'] ) ? absint( $_GET['radius'] ) : 50;

		// Radius of the earth 3959 miles or 6371 kilometers.
		$earth_radius = 'mi' == $va_options->google_maps_unit ? 3959 : 6371;

		// Use the Haversine formula to compute great circle distances between two pairs of coordinates on a sphere.
		// Basically find any listings that are within a certain radius of the submitted location (lat and lng).
		$sql = $wpdb->prepare( "
			SELECT p.ID, lat.meta_value, lng.meta_value,
			( %d * acos(
				cos( radians( %s ) )
				* cos( radians( lat.meta_value ) )
				* cos( radians( lng.meta_value ) - radians( %s ) )
				+ sin( radians( %s ) )
				* sin( radians( lat.meta_value ) )
			) )
			AS distance
			FROM $wpdb->posts p
			INNER JOIN $wpdb->postmeta lat ON p.ID = lat.post_id
			INNER JOIN $wpdb->postmeta lng ON p.ID = lng.post_id
			WHERE 1 = 1
			AND p.post_status = 'publish'
			AND lat.meta_key  = 'geo_lat'
			AND lng.meta_key  = 'geo_lng'
			HAVING distance <= %s
			ORDER BY distance ASC",
			$earth_radius,
			$lat,
			$lng,
			$lat,
			$radius
		);

		// Send back the results. Make post_ids the array key.
		return $wpdb->get_results( $sql, OBJECT_K );
	}


	/**
	 * Generates the admin option fields.
	 *
	 * Remember to add each field to the whitelist otherwise it won't save.
	 * Located in includes/options.php
	 *
	 * @since  4.0.0
	 *
	 * @return array
	 */
	public function admin_fields( $page ) {
		self::$page = $page;

		// Insert the tab.
		$page->tabs->add( 'google_maps', __( 'Maps', APP_TD ) );

		// Insert the fields.
		$page->tab_sections['google_maps']['general'] = array(
			'title'  => __( 'Google Maps', APP_TD ),
			'fields' => array(
				array(
					'title'   => __( 'Region Biasing', APP_TD ),
					'type'    => 'text',
					'default' => 'US',
					'name'    => 'google_maps_region',
					'desc'    => sprintf( __( 'Find your two-letter <a href="%s" target="_blank">region code</a>', APP_TD ), 'http://en.wikipedia.org/wiki/ISO_3166-1#Current_codes' ),
					'tip'     => __( "If you set this to 'IT' and a user enters 'Florence' in the location search field, it will target 'Florence, Italy' rather than 'Florence, Alabama'.", APP_TD ),
					'extra'   => array(
						'class' => 'small-text',
					)
				),
				array(
					'title'   => __( 'Language', APP_TD ),
					'type'    => 'text',
					'default' => get_locale() ? substr( get_locale(), 0, 2 ) : '',
					'name'    => 'google_maps_language',
					'desc'    => sprintf( __( 'Find your two-letter <a href="%s" target="_blank">language code</a>', APP_TD ), 'http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes' ),
					'tip'     => __( 'Used to format the address and map controls.', APP_TD ),
					'extra'   => array(
						'class' => 'small-text',
					)
				),
				array(
					'title'  => __( 'Distance Unit', APP_TD ),
					'type'   => 'select',
					'name'   => 'google_maps_unit',
					'values' => array(
						'mi' => __( 'Miles', APP_TD ),
						'km' => __( 'Kilometers', APP_TD ),
					),
					'tip'    => '',
				),
				array(
					'title'  => __( 'Marker Color', APP_TD ),
					'type'   => 'custom',
					'name'   => 'google_maps_marker_color',
					'render' => array( $this, 'va_color_picker_callback' ),
					'tip'    => '',
				),
				array(
					'title'  => __( 'Color Scheme', APP_TD ),
					'type'   => 'select',
					'name'   => 'google_color_scheme',
					'values' => array(
						'standard'           => __( 'Standard', APP_TD ),
						'apple'              => __( 'Apple', APP_TD ),
						'blue-hues'          => __( 'Blue Hues', APP_TD ),
						'blue-water'         => __( 'Blue Water', APP_TD ),
						'blue-essence'       => __( 'Blue Essence', APP_TD ),
						'cream'              => __( 'Cream', APP_TD ),
						'light-dream'        => __( 'Light Dream', APP_TD ),
						'midnight-commander' => __( 'Midnight Commander', APP_TD ),
						'night'              => __( 'Night', APP_TD ),
						'pale-dawn'          => __( 'Pale Dawn', APP_TD ),
						'paper'              => __( 'Paper', APP_TD ),
						'red-hues'           => __( 'Red Hues', APP_TD ),
						'retro'              => __( 'Retro', APP_TD ),
						'subtle-grayscale'   => __( 'Subtle Grayscale', APP_TD ),
						'ultra-light'        => __( 'Ultra Light', APP_TD ),
					),
					'tip'    => sprintf( __( 'Select a preset color scheme for the maps. If you are a developer, use your own .json color scheme via the "appthemes_google_map_styles_file" filter. Download a preset from <a href="%s" target="_blank">Snazzy Maps</a> or build your own with the <a href="%s" target="_blank">Google Maps Styling Wizard</a>.', APP_TD ), 'https://snazzymaps.com', 'https://mapstyle.withgoogle.com/' ),
				),
				array(
					'title' => __( 'API Key', APP_TD ),
					'desc'  => sprintf( __( 'Follow our <a href="%s" target="_blank">Google API key tutorial</a> and <a href="%s">verify it works</a>.', APP_TD ), 'https://docs.appthemes.com/tutorials/setting-google-maps-api-key/', esc_url( admin_url( 'admin.php?page=app-system-info' ) ) ),
					'type'  => 'text',
					'name'  => 'google_maps_api_key',
					'tip'   => sprintf( __( 'Enter your Google API key. This field is required for address geocoding and maps to work. <a href="%s" target="_blank">API request limits</a>.', APP_TD ), 'https://developers.google.com/maps/documentation/javascript/usage#usage-limits-for-google-maps-javascript-api-services' ),
				),
			),
		);

	}

	/**
	 * Custom callback to generate the color picker.
	 *
	 * @since   4.0.0
	 *
	 * @return  string
	 */
	public function va_color_picker_callback() {
		global $va_options;

		wp_enqueue_script( 'wp-color-picker' );
		wp_enqueue_style( 'wp-color-picker' );

		$value = $va_options->google_maps_marker_color;

		ob_start();
		?>
		<input type="text" class="small-text" id="google_maps_marker_color" name="google_maps_marker_color" data-default-color="<?php echo $value; ?>" value="<?php echo $value; ?>" />
		<script>
			jQuery( document ).ready( function( $ ){
				$( 'input[name="google_maps_marker_color"]' ).wpColorPicker();
			});
		</script>
	<?php
		return ob_get_clean();
	}

	/**
	 * Include inline css if custom colors have been set.
	 *
	 * @since   4.0.0
	 *
	 * @return  string Inline css
	 */
	public function inline_css() {
		global $va_options;

		$css = '
			/**
			 * AppThemes Google Maps Module
			 */
			.map-rich-marker {
				color: %1$s;
			}
			i.fa.fa-circle:after {
				border-color: %1$s rgba(0, 0, 0, 0) rgba(0, 0, 0, 0);
			}
		';

		return sprintf( $css, $va_options->google_maps_marker_color );
	}

	/**
	 * Display admin notices based on missing required fields.
	 *
	 * @since 4.0.0
	 *
	 * @return string The admin notice message.
	 */
	public function admin_notices() {
		global $va_options;

		$handle = 'appthemes_google_maps_api_notice';

		// If the API key has already been entered, bail.
		if ( ! empty( $va_options->google_maps_api_key ) || ( get_option( $handle . '_dismissed', false ) ) ) {
			return;
		}

		echo scb_admin_notice( sprintf( __( 'Google Maps, GeoCoding, and Places (search autocomplete) will not work correctly with %s until a valid API key has been entered. <a href="%s">Add an API key &rarr;</a>', APP_TD ), get_option( 'current_theme' ), admin_url( 'admin.php?page=app-settings&tab=google_maps' ) ), $handle . ' notice notice-error is-dismissible' );
		echo '<span id="' . $handle . '_nonce" style="display:none;">' . wp_create_nonce( $handle . '_' . get_current_user_id() ) . '</span>';
	}
}
