<?php
/**
 * Setup the menus.
 *
 * @package Vantage
 * @since   4.0.0
 */

/**
 * Register menus.
 *
 * @since 4.0.0
 */
register_nav_menus( array(
	'primary'   => __( 'Primary Menu', APP_TD ),
	'secondary' => __( 'Secondary Menu', APP_TD ),
	'footer'    => __( 'Footer Menu', APP_TD ),
) );

if ( ! function_exists( 'va_header_menu_primary' ) ) :
/**
 * Setup the primary header menu.
 *
 * @since 4.0.0
 */
function va_header_menu_primary() {
	wp_nav_menu( array(
		'theme_location'  => 'primary',
		'container_class' => 'top-bar-right',
		'menu_class'      => 'menu medium-horizontal vertical',
		'items_wrap'      => '<ul id="%1$s" class="%2$s" data-responsive-menu="accordion medium-dropdown">%3$s</ul>',
		'fallback_cb'     => false,
		'depth'           => 5,
		'walker'          => new VA_Topbar_Menu_Walker(),
	) );
}
endif;

if ( ! function_exists( 'va_header_menu_secondary' ) ) :
/**
 * Setup the secondary header menu.
 *
 * @since 4.0.0
 */
function va_header_menu_secondary() {
	wp_nav_menu( array(
		'theme_location' => 'secondary',
		'container'      => false,
		'menu_class'     => 'menu medium-horizontal vertical',
		'items_wrap'     => '<ul id="%1$s" class="%2$s" data-responsive-menu="accordion medium-dropdown">%3$s</ul>',
		'fallback_cb'    => false,
		'depth'          => 5,
		'walker'         => new VA_Topbar_Menu_Walker(),
	));
}
endif;

if ( ! function_exists( 'va_footer_menu' ) ) :
/**
 * Setup the footer menu.
 *
 * @since 4.0.0
 */
function va_footer_menu() {
	wp_nav_menu( array(
		'theme_location'  => 'footer',
		'container'       => false,
		'menu_class'      => 'social-media list-inline',
		'fallback_cb'     => false,
		'depth'           => 1,
	) );
}
endif;

/**
 * Add additional login & register items to the primary header menu.
 *
 * @since 4.0.0
 *
 * @todo Use actual menu pages instead of hardcoded items.
 *
 * @param string $items The HTML list content for the menu items.
 * @param object $args  An object containing wp_nav_menu() arguments.
 */
function va_header_menu_primary_items_login( $items, $args ) {

	if ( is_user_logged_in() ) {
		return $items;
	}

	$fields = array();
	$output = '';

	if ( get_option( 'users_can_register' ) ) {
		$fields['register'] = '<li id="menu-item-register"><a href="' . esc_url( wp_registration_url() ) . '" rel="nofollow" class="button hollow" id="register-button-link">' . __( 'Signup', APP_TD ) . '</a></li>';
	}

	$fields['login'] = '<li id="menu-item-login"><a href="' . esc_url( wp_login_url( get_permalink() ) ) . '" rel="nofollow" class="button" id="login-button-link">' . __( 'Login', APP_TD ) . '</a></li>';

	/**
	 * Filters the default user login menu items.
	 *
	 * @since 4.0.0
	 *
	 * @param array $fields The default user account fields.
	 */
	$fields = apply_filters( 'va_header_menu_primary_items_login', $fields );

	// Add the user login submenu items.
	foreach ( $fields as $field ) {
		$output .= $field;
	}

	return $items . $output;
}
add_filter( 'wp_nav_menu_primary_items', 'va_header_menu_primary_items_login', 10, 2 );

/**
 * Add user links to the primary top bar dropdown menu.
 *
 * Uses the wp_nav_menu_{$menu->slug}_items filter.
 *
 * @since 4.0.0
 *
 * @todo Use actual menu pages instead of hardcoded items.
 *
 * @param string $items The HTML list content for the menu items.
 * @param object $args  An object containing wp_nav_menu() arguments.
 */
function va_header_menu_primary_user( $items, $args ) {

	$output = $user_menu = '';

	if ( is_user_logged_in() ) {

		// Get the user object.
		$user = wp_get_current_user();

		/**
		 * Filter the display name next to the gravatar.
		 *
		 * @since 4.0.0
		 *
		 * @param string $user->user_login The user login field.
		 * @param array $user The user object.
		 */
		$display_name = apply_filters( 'va_user_menu_display_name', $user->user_login, $user );
		$avatar       = get_avatar( $user->user_email, 30, '', '', array( 'class' => 'img-circle' ) );

		// Need to build the menu wrapper so we can create the avatar dropdown.
		$user_menu  = '<li id="menu-item-username" class="menu-item has-submenu"><a href="#" class="user-gravatar">' . $avatar . $display_name . '</a><ul id="menu-user" class="menu submenu vertical" data-submenu>';

		// Get the user submenu items.
		$submenus = va_header_menu_primary_items_user();

		// Add the user submenu items.
		foreach ( $submenus as $submenu ) {
			$user_menu .= $submenu;
		}

		$user_menu .= '</ul></li>';

		/**
		 * Fires in the primary menu under the users' avatar dropdown.
		 *
		 * @since 4.0.0
		 *
		 * @param string $user_menu The user menu dropdown list items.
		 */
		$user_menu = apply_filters( 'va_header_menu_primary_user', $user_menu );
	}

	return $items . $user_menu;
}
add_filter( 'wp_nav_menu_primary_items', 'va_header_menu_primary_user', 40, 2 );

/**
 * Filters the primary header nav menu so we can dynamically replace the
 * `{{user_account}}` title placeholder with the user gravatar.
 *
 * @todo Use this instead of va_header_menu_primary_user() so user account menu
 *       can be moved around via menus. Need to figure out best way to create logout
 *       page and dynamically create other user account page menu items.
 *
 * @since 4.x.x
 *
 * @param string $item_output The menu item's starting HTML output.
 * @param object $item        Menu item data object.
 * @param int    $depth       Depth of menu item. Used for padding.
 * @param array  $args        An array of wp_nav_menu() arguments.
 *
 * @return string $item_output
 */
function va_header_menu_primary_user_NEW( $item_output, $item, $depth, $args ) {

	$placeholder = '{{user_account}}';

	if ( $placeholder != $item->title ) {
		return $item_output;
	}

	if ( is_user_logged_in() ) {

		$user = wp_get_current_user();

		/**
		 * Filter the display name next to the gravatar.
		 *
		 * @since 4.0.0
		 *
		 * @param string $user->user_login The user login field.
		 * @param array $user The user object.
		 */
		$display_name = apply_filters( 'va_user_menu_display_name', $user->user_login, $user );
		$avatar       = get_avatar( $user->user_email, 30, '', '', array( 'class' => 'img-circle' ) ) . $display_name;

		$item_output = str_replace( $placeholder, $avatar, $item_output );
	}

	return $item_output;
}
//add_filter( 'walker_nav_menu_start_el', 'va_header_menu_primary_user_NEW', 10, 4 );

/**
 * Builds the logged in users drop-down menu items.
 *
 * @since   4.0.0
 *
 * @return  string
 */
function va_header_menu_primary_items_user() {

	if ( ! is_user_logged_in() ) {
		return;
	}

	$user = wp_get_current_user();

	$fields = array(
		'listings'   => '<li id="menu-item-my-listings"><a href="' . esc_url( va_dashboard_url( 'listings', $user->ID, true ) ) . '" class="user-listings"><i class="fa fa-list" aria-hidden="true"></i>' . __( 'Listings', APP_TD ) . '</a></li>',
		'favorites'  => '<li id="menu-item-my-favorites"><a href="' . esc_url( va_dashboard_url( 'favorites', $user->ID, true ) ) . '" class="user-favorites"><i class="fa fa-heart"   aria-hidden="true"></i>' . __( 'Favorites', APP_TD ) . '</a></li>',
		'profile'    => '<li id="menu-item-edit-profile"><a href="' . esc_url( appthemes_get_edit_profile_url() ) . '" class="user-edit-profile"><i class="fa fa-user" aria-hidden="true"></i>' . __( 'Edit Profile', APP_TD ) . '</a></li>',
		'divider'    => '<li class="divider"></li>',
		'logout'     => '<li id="menu-item-logout"><a href="' . esc_url( wp_logout_url( get_permalink() ) ) . '" class="user-logout"><i class="fa fa-sign-out" aria-hidden="true"></i>' . __( 'Log out', APP_TD ) . '</a></li>',
	);

	/**
	 * Filters the default user account menu items.
	 *
	 * @since 4.0.0
	 *
	 * @param array $fields The default user account fields.
	 */
	return apply_filters( 'va_header_menu_primary_items_user', $fields );
}
