<?php
/**
 * Social meta functions
 *
 * @todo Put all functions into a class.
 *
 * @package Vantage\Social
 *
 * @since   1.4.0
 * @since   4.0.0 Added comment blocks. Combined similar functions.
 */

/**
 * Re-register social networks to remove default tooltip and change title.
 */
APP_Social_Networks::register_network( 'google-plus', array(
	'title' => __( 'Google+', APP_TD ),
	'tip'      => '',
	'base_url' => 'https://plus.google.com/',
	'user_url' => 'https://plus.google.com/%s/',
) );

APP_Social_Networks::register_network( 'facebook', array(
	'title' => __( 'Facebook', APP_TD ),
	'tip'   => '',
) );

APP_Social_Networks::register_network( 'twitter', array(
	'title' => __( 'Twitter', APP_TD ),
	'tip'   => '',
) );

APP_Social_Networks::register_network( 'instagram', array(
	'title' => __( 'Instagram', APP_TD ),
	'tip'   => '',
) );

APP_Social_Networks::register_network( 'youtube', array(
	'title' => __( 'YouTube', APP_TD ),
	'tip'      => '',
	'user_url' => 'https://youtube.com/user/%s/',
) );

APP_Social_Networks::register_network( 'pinterest', array(
	'title' => __( 'Pinterest', APP_TD ),
	'tip'   => '',
) );


/**
 * Returns the title for the given social network
 *
 * @since 1.0.0
 *
 * @param string $social_network
 *
 * @return string social network name
 */
function va_get_social_network_title( $social_network ) {
	return APP_Social_Networks::get_title( $social_network );
}

/**
 * Returns the tip for filling the given social network
 *
 * @since 1.0.0
 *
 * @param string $social_network
 *
 * @return string social network tip
 */
function va_get_social_network_tip( $social_network ) {
	return APP_Social_Networks::get_tip( $social_network );
}

/**
 * Returns the user account URL for the given social network
 *
 * @since 1.0.0
 *
 * @param type $social_network
 * @param type $account
 *
 * @return string Escaped URL
 */
function va_get_social_account_url( $social_network, $account = '' ) {
	return APP_Social_Networks::get_url( $social_network, $account );
}

/**
 * Retrieves an array of allowed social networks to be refined for each type of object
 *
 * @since 1.0.0
 * @since 4.0.0 Renamed and added new filter.
 *
 * @return array An array of allowed registered social networks
 */
function va_allowed_social_networks() {
	$networks = array(
		'google-plus',
		'facebook',
		'twitter',
		'instagram',
		'youtube',
		'pinterest',
	);

	/**
	 * Filter the whitelist of social networks.
	 *
	 * @since 4.0.0
	 *
	 * @param array $networks A list of all whitelisted social networks.
	 */
	return apply_filters( 'va_allowed_social_networks', $networks );
}

/**
 * Get all user account social networks.
 *
 * @since 1.0.0
 */
function va_get_allowed_user_networks() {
	/**
	 * Filter the whitelist of user account social networks.
	 *
	 * @since 1.0.0
	 *
	 * @param array A list of all whitelisted social networks.
	 */
	return apply_filters( 'va_user_allowed_social_networks', va_allowed_social_networks() );
}

/**
 * Get all listing post type social networks.
 *
 * @since 1.0.0
 */
function va_get_allowed_listing_networks() {
	/**
	 * Filter the whitelist of listing post type social networks.
	 *
	 * @since 1.0.0
	 *
	 * @param array A list of all whitelisted social networks.
	 */
	return apply_filters( 'va_listing_allowed_social_networks', va_allowed_social_networks() );
}

/**
 * Get all listing post type contact methods including social networks, email,
 * phone, address and website.
 *
 * @since 4.0.0
 *
 * @return array A list of all whitelisted contact methods.
 */
function va_get_allowed_contact_listing_methods() {
	$social = va_get_allowed_listing_networks();
	$other  = array( 'address', 'phone', 'website', 'email' );

	return array_merge( $other, (array) $social );
}

/**
 * Get all user social networks.
 *
 * @since 1.0.0
 * @since 4.0.0 Rebuilt so it's cleaner.
 *
 * @param boolean $remove_empty Removes any social networks from array if value is empty.
 *
 * @return array $fields Key value pair of user social network name and url.
 */
function va_get_available_user_networks( $remove_empty = false ) {
	global $user_ID;

	$fields = array();

	foreach ( va_get_allowed_user_networks() as $key => $value ) {
		$fields[ $value ] = get_user_meta( $user_ID, $value, true );
	}

	if ( true === $remove_empty ) {
		$fields = array_filter( $fields );
	}

	return $fields;
}

/**
 * Get all listing post type social networks.
 *
 * @since 1.0.0
 * @since 4.0.0 Rebuilt so it's cleaner.
 *
 * @param boolean $remove_empty Removes any social networks from array if value is empty.
 *
 * @return array $fields Key value pair of listing social network name and url.
 */
function va_get_available_listing_networks( $remove_empty = false ) {
	global $post;

	$fields = array();

	foreach ( va_get_allowed_listing_networks() as $key => $value ) {
		$fields[ $value ] = get_post_meta( $post->ID, $value, true );
	}

	if ( true === $remove_empty ) {
		$fields = array_filter( $fields );
	}

	return $fields;
}
