<?php
/**
 * Structured data markup classes.
 *
 * @package Components\StructuredData
 * @author AppThemes
 * @since 1.0.0
 */

if ( ! class_exists( 'APP_Structured_Data' ) ) :
/**
 * Main class to output json-ld schema.org markup.
 *
 * @since 1.0.0
 */
final class APP_Structured_Data {

	/**
	 * Holds the payload to output json-ld.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 *
	 * @since 4.0.0
	 */
	public $data = array();

	/**
	 * Holds an instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	private static $instance;

	/**
	 * Singleton implementation to load construct only once.
	 *
	 * @since 1.0.0
	 *
	 * @return object
	 */
	public static function instance() {

		if ( isset( self::$instance ) ) {
			return self::$instance;
		}

		self::$instance = new self;
	}

	/**
	 * Constructor
	 */
	private function __construct() {
		add_action( 'init',    array( $this, 'require_files' ) );
		add_action( 'wp_head', array( $this, 'build_schema' ) );
	}

	/**
	 * Load all schema type class files.
	 *
	 * @since 1.0.0
	 */
	public function require_files() {
		foreach ( glob( dirname( __FILE__ ) . '/types/*.php' ) as $file ) {
			require_once( $file );
		}
	}

	/**
	 * Build the json-ld schema.org output based on page being viewed.
	 *
	 * Main logic method sub-classes should override in their code.
	 *
	 * @since 1.0.0
	 */
	public function build_schema() {

		/**
		 * Fires at the end of the build schema method before output.
		 *
		 * @since 1.0.0
		 */
		do_action( 'appthemes_schema_build_data' );

		// Send everything to the final output method.
		$this->output();
	}

	/**
	 * Adds the organization json-ld schema.
	 *
	 * @since 1.0.0
	 *
	 * @return array
	 */
	public function organization_home() {

		$organization['@context'] = 'http://schema.org';

		$output = array_merge( $organization, APP_Schema_Type_Organization::type() );

		/**
		 * Filters the organization home schema output.
		 *
		 * @since 1.0.0
		 *
		 * @param array $output The organization home schema output.
		 */
		return apply_filters( 'appthemes_schema_organization_home', $output );
	}

	/**
	 * Add blog post comments to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $output List of BlogPosting type schema properties.
	 * @return array
	 */
	public function add_comments( $output ) {

		if ( ! is_array( $comments = APP_Schema_Type_Comment::type() ) ) {
			return $output;
		}

		// Set the array key.
		$r['comment'] = $comments;

		return array_merge( $output, $r );
	}

	/**
	 * Add blog post tags to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $output List of BlogPosting type schema properties.
	 * @return array
	 */
	public function add_tags( $output ) {

		// Make sure we've got tags returned.
		if ( false !== array_search( '', $tags = $this->terms() ) ) {
			return $output;
		}

		return array_merge( $output, $tags );
	}

	/**
	 * Add blog categories to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $output List of BlogPosting type schema properties.
	 * @return array
	 */
	public function add_categories( $output ) {

		// Category terms comma separated.
		if ( ! is_array( $categories = $this->terms( 'category', 'genre', ', ' ) ) ) {
			return $output;
		}

		return array_merge( $output, $categories );
	}

	/**
	 * Add the blog post word count to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $output List of BlogPosting type schema properties.
	 * @return array
	 */
	public function add_word_count( $output ) {

		if ( ! is_array( $word_count = $this->word_count() ) ) {
			return $output;
		}

		return array_merge( $output, $word_count );
	}

	/**
	 * Add any type of taxonomy to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param string $taxonomy  The taxonomy we want to use.
	 * @param string $property  The schema property.
	 * @param string $delimiter The schema property delimiter.
	 * @return array
	 */
	public function terms( $taxonomy = 'post_tag', $property = 'keywords', $delimiter = ' ' ) {
		global $post;

		// Get all the terms.
		$terms = get_the_terms( $post->ID, $taxonomy );

		// No terms found.
		if ( empty( $terms ) ) {
			return array();
		}

		// Grab all the term names.
		$term_names = wp_list_pluck( $terms, 'name' );

		// Break apart array into spaced-separated string.
		$term_names = implode( $delimiter, (array) $term_names );

		// Set the array key.
		$output[$property] = $term_names;

		/**
		 * Filters the terms array.
		 *
		 * @since 1.0.0
		 *
		 * @param array $output The term names array key and values.
		 * @param array $terms  The terms array.
		 */
		return apply_filters( 'appthemes_schema_terms', $output, $terms );
	}

	/**
	 * Add the person content for the author page to the output filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $output List of Person type schema properties.
	 * @return array
	 */
	public function author() {

		$r['@context'] = 'http://schema.org';

		$author = get_user_by( 'id', get_query_var( 'author' ) );

		return array_merge( $r, APP_Schema_Type_Person::type( get_the_author_meta( 'display_name', $author->ID ) ) );
	}

	/**
	 * Count the total number of words in the post.
	 *
	 * @since 1.0.0
	 *
	 * @return int The count of total words.
	 */
	public function word_count() {
		global $post;

		// We only want text to count. No html, script, etc tags.
		$count = str_word_count( wp_strip_all_tags( $post->post_content ) );

		// Set the array key.
		$word_count['wordcount'] = $count;

		/**
		 * Filters the word count array.
		 *
		 * @since 1.0.0
		 *
		 * @param array $word_count The word count array key and values.
		 */
		return apply_filters( 'appthemes_schema_wordcount', $word_count );
	}

	/**
	 * Output the final json-ld code and display it.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function output() {

		/**
		 * Filters the final data output before delivering it to the browser.
		 *
		 * @since 1.0.0
		 *
		 * @param array $this->data The entire data payload of schema.org output.
		 */
		$data = apply_filters( 'appthemes_schema_output', $this->data );

		if ( is_array( $data ) && ! empty( $data ) ) {

			if ( defined( 'SCRIPT_DEBUG' ) && version_compare( PHP_VERSION, '5.4.0', '>=' ) ) {
				$output = wp_json_encode( $data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT );
			} else {
				$output = wp_json_encode( $data );
			}

			echo '<!-- Start AppThemes json-ld structured data -->', PHP_EOL;
			echo '<script type="application/ld+json">', PHP_EOL;
			echo $output, PHP_EOL;
			echo '</script>', PHP_EOL;
			echo '<!-- End AppThemes json-ld structured data -->', PHP_EOL;
		}

		// Empty the array so we don't output it multiple times.
		$this->data = array();
	}
}

APP_Structured_Data::instance();

endif;
