<?php
/**
 * Functions used in the Templates and Loops
 *
 * @package Vantage\TemplateTags
 * @since   Vantage 1.0
 */

/**
 * Displays listing address.
 *
 * @since 1.0.0
 * @since 4.0.0 Added support for new geo coding.
 *
 * @return string
 */
function the_listing_address() {
	global $post;

	// Legacy support.
	if ( $address = get_post_meta( $post->ID, 'address', true ) ) {
		return $address;
	} else {
		$geo_values = APP_Google_Geocoding::get_post_meta( $post->ID );
		// If it's been geocoded, return better structure, otherwise return user entered address.
		if ( ! empty( $geo_values['lat'] ) ) {
			return '<span class="listing-street">' . $geo_values['street_number'] . ' ' . $geo_values['street'] . '</span> <span class="listing-city-state-zip">' . $geo_values['city'] . ', ' . $geo_values['state_short'] . ' ' . $geo_values['postal_code'] . '</span>';
		} else {
			return '<span class="listing-address-raw">' . $geo_values['raw_address'] . '</span>';
		}
	}
}


/**
 * Displays listing categories.
 *
 * @since 1.0.0
 *
 * @param int $listing_id (optional).
 * @return void
 */
function the_listing_categories( $listing_id = 0 ) {

	$listing_id = $listing_id ? $listing_id : get_the_ID();

	$cats = get_the_listing_categories( $listing_id );
	if ( ! $cats ) {
		return;
	}

	$_cats = array();

	foreach ( $cats as $cat ) {
		$_cats[] = html_link( get_term_link( $cat ), $cat->name );
	}

	$cats_list = implode( ' ', $_cats );

	echo $cats_list;
}


/**
 * Displays the listing custom fields.
 *
 * @since 1.0.0
 * @since 4.0.0 Use global $post instead of parameter.
 *
 * @return string|boolean
 */
function get_listing_custom_fields() {
	global $post;

	$exclude = array(
		'type' => array( 'tax_input', 'file' ),
		'name' => array( 'post_title', 'post_content' ),
	);

	$exclude['name'] = array_merge( $exclude['name'], va_get_allowed_contact_listing_methods() );

	$format = '<tr id="%id%"><td class="listing-custom-field-title">%title%</td><td class="listing-custom-field-value">%value%</td></tr>';

	$fields = appthemes_listing_details( $post->ID, array(), $exclude, $format );

	if ( ! $fields ) {
		return;
	}

	return html( 'table', array( 'class' => 'listing-custom-fields' ), html( 'tbody', $fields ) );
}

if ( ! function_exists( 'va_entry_meta' ) ) :
/**
 * Prints HTML with meta information for the categories, tags.
 *
 * @since 4.0.0
 */
function va_entry_meta() {

	$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';

	if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) ) {
		$time_string = '<time class="entry-date published" datetime="%1$s">%2$s</time><meta content="%3$s">';
	}

	$time_string = sprintf( $time_string,
		esc_attr( get_the_date( 'c' ) ),
		esc_html( get_the_date() ),
		esc_attr( get_the_modified_date( 'c' ) )
	);

	$posted_on = sprintf(
		esc_html_x( 'Posted on %s', 'post date', APP_TD ), $time_string
	);

	$byline = sprintf(
		esc_html_x( 'by %s', 'post author', APP_TD ),
		'<span class="author"><a class="url fn n" href="' . esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ) . '">' . esc_html( get_the_author() ) . '</a></span>'
	);

	echo '<span class="posted-on">' . $posted_on . '</span><span class="byline"> ' . $byline . '</span>';
}
endif;

if ( ! function_exists( 'va_entry_small_meta' ) ) :
/**
 * Prints HTML with meta information for the blog main page.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_entry_small_meta() {
?>
<ul class="meta-list list-inline">
	<li class="post-author"><?php echo get_avatar( get_the_author_meta( 'user_email' ), 24 ); ?></li>
	<li class="post-time post_date date updated"><?php echo get_the_date(); ?></li>
	<li class="post-comments fa-icon fa-comments"><span class="comments-count"><?php echo get_comments_number(); ?></span></li>
</ul><!-- .meta-list -->
<?php
}
endif;


/**
 * Returns listing categories ids.
 *
 * @since 1.0.0
 *
 * @param int $listing_id (optional).
 * @return array
 */
function get_the_listing_categories( $listing_id = 0 ) {
	$listing_id = $listing_id ? $listing_id : get_the_ID();

	$_terms = get_the_terms( $listing_id, VA_LISTING_CATEGORY );

	if ( ! $_terms ) {
		return array();
	}

	// WordPress does not always key with the term_id, but thats what we want for the key.
	$terms = array();
	foreach ( $_terms as $_term ) {
		$terms[ $_term->term_id ] = $_term;
	}

	return $terms;
}

/**
 * Displays edit listing link.
 *
 * @since 4.0.0
 *
 * @return void
 */
function va_edit_listing_link() {
	global $post;

	// Only show if user can edit listing.
	if ( ! current_user_can( 'edit_post', $post->ID ) ) {
		return;
	}

	$link = '<a href="' . esc_url( va_get_listing_edit_url( $post->ID ) ) . '" class="listing-edit-link listing-icon"><i class="fa fa-pencil"></i></a>';

	/**
	 * Filter the edit listing link.
	 *
	 * @since 4.0.0
	 */
 	echo apply_filters( 'va_edit_listing_link', $link, $post );

}
add_action( 'va_listing_single_bar_actions', 'va_edit_listing_link', 40 );

/**
 * Displays listing purchase link.
 *
 * @since 4.0.0
 *
 * @param int    $listing_id (optional).
 * @param string $text (optional).
 * @return void
 */
function va_upgrade_listing_link() {
	global $post, $va_options;

	$listing = APP_Listing_Director::get( $post->post_type );
	$url     = va_get_listing_purchase_url( $post->ID );

	// Don't show if we aren't charging.
	if ( ! $listing->options->charge || ! $listing->addons || ! $url ) {
		return;
	}

	$addons = $listing->addons->get_addons();

	// Check if option is enabled.
	if ( empty( $addons ) ) {
		return;
	}

	/* @var $plans_mod APP_Listing_Plans_Registry */
	$plans_mod = $listing->plans;
	$plan      = $plans_mod->get_current_plan( $post->ID );

	if ( ! $plan->is_upgradable() ) {
		return;
	}

	$link = '<a href="' . esc_url( $url ) . '" class="listing-upgrade-link button">' . __( '<i class="fa fa-level-up"></i> Upgrade Listing', APP_TD ) . '</a>';

	/**
	 * Filter the upgrade listing link.
	 *
	 * @since 4.0.0
	 */
 	echo apply_filters( 'va_upgrade_listing_link', $link, $post );
}
add_action( 'va_listing_single_head_actions', 'va_upgrade_listing_link', 30 );

/**
 * Displays listing renew link.
 *
 * @param int    $listing_id (optional).
 * @param string $text (optional).
 * @return void
 */
function the_listing_renew_link( $listing_id = 0, $text = '' ) {
	$listing_id = $listing_id ? $listing_id : get_the_ID();

	if ( ! current_user_can( 'edit_post', $listing_id ) ) {
		return;
	}

	if ( empty( $text ) ) {
		$text = __( 'Renew', APP_TD );
	}

	echo html( 'a', array(
		'class' => 'listing-edit-link listing-renew-link',
		'href' => va_get_listing_renew_url( $listing_id ),
	), $text );
}

/**
 * Display the listing claimable link.
 *
 * @since 4.0.0
 *
 * @return void
 */
function va_claim_listing_link() {
	global $post;

	// Don't show if the listing isn't allowed to be claimed.
	if ( ! get_post_meta( $post->ID, 'listing_claimable', true ) ) {
		return;
	}

	// Don't show if a claim is already pending.
	if ( 'pending-claimed' == get_post_status( $post->ID ) ) {
		return;
	}

	$claim_link = '';
	$claim_url  = va_get_listing_claim_url( $post->ID );

	if ( $claim_url ) {
		$claim_link = '<a href="' . esc_url( $claim_url ) . '" class="listing-claim-link button">' . __( '<i class="fa fa-check-circle-o"></i> Claim Listing', APP_TD ) . '</a>';
	}

	/**
	 * Filter the claim listing link.
	 *
	 * @since 4.0.0
	 */
	echo apply_filters( 'va_claim_listing_link', $claim_link, $post );
}
add_action( 'va_listing_single_head_actions', 'va_claim_listing_link' );

/**
 * Returns listing edit url.
 *
 * @param int $listing_id
 * @return string
 */
function va_get_listing_edit_url( $listing_id ) {
	return appthemes_get_process_url( VA_LISTING_PTYPE, 'edit', $listing_id );
}

/**
 * Returns listing renew url.
 *
 * @param int $listing_id
 * @return string
 */
function va_get_listing_renew_url( $listing_id ) {
	return appthemes_get_process_url( VA_LISTING_PTYPE, 'renew', $listing_id );
}

/**
 * Returns listing purchase url.
 *
 * @since 1.0.0
 *
 * @param int $listing_id
 * @return string
 */
function va_get_listing_purchase_url( $listing_id ) {
	return appthemes_get_process_url( VA_LISTING_PTYPE, 'upgrade', $listing_id );
}

/**
 * Returns listing claim url.
 *
 * @param int $listing_id
 *
 * @return string
 */
function va_get_listing_claim_url( $listing_id ) {
	return appthemes_get_process_url( VA_LISTING_PTYPE, 'claim', $listing_id );
}

/**
 * Returns a listing create url.
 *
 * @since 1.0.0
 *
 * @return string
 */
function va_get_listing_create_url() {
	return appthemes_get_process_url( VA_LISTING_PTYPE, 'new', $listing_id );
}

/**
 * Displays field for refine listings by category.
 *
 * @since 3.0.0
 *
 * @param array $args Array of the default options. Default empty.
 * @return void
 */
function va_refine_categories( $args = array() ) {

	require_once ABSPATH . '/wp-admin/includes/template.php';

	$defaults = array(
		'taxonomy'    => VA_LISTING_CATEGORY,
		'request_var' => 'listing_cat',
		'class'       => 'refine-categories-list',
	);

	$args = wp_parse_args( $args, $defaults );

	// Get listing cat id or set to zero.
	$tax_id = absint( is_tax() ? get_queried_object()->term_id : 0 );

	// Check the listing cats if they were searched or viewing a listing cat archive page.
	$selected = isset( $_GET[ $args['request_var'] ] ) ? $_GET[ $args['request_var'] ] : array( $tax_id );

	ob_start();

	wp_terms_checklist( 0, array(
		'taxonomy'      => $args['taxonomy'],
		'selected_cats' => $selected,
		'checked_ontop' => false,
	) );

	$output = ob_get_clean();
	$output = str_replace( 'tax_input[' . $args['taxonomy'] . ']', $args['request_var'], $output );
	$output = str_replace( 'disabled=\'disabled\'', '', $output );
	$output = '<ul class="' . $args['taxonomy'] . '">' . $output . '</ul>' . "\n";

	/**
	 * Filters the HTML output of listing categories as checkboxes.
	 *
	 * @since 4.0.0
	 *
	 * @param string $output HTML output for checkbox list of categories.
	 * @param array  $args Array of the default options.
	 */
	$html = apply_filters( 'va_refine_categories', $output, $args );

	echo $html;
}

/**
 * Outputs javascript redirect.
 *
 * @since 1.0.0
 *
 * @param string $url
 * @return void
 */
function va_js_redirect( $url ) {
	echo html( 'a', array( 'href' => esc_url( $url ) ), __( 'Continue', APP_TD ) );
	echo html( 'script', 'location.href="' . esc_url_raw( $url ) . '"' );
}

/**
 * Returns the lat and long of the listing.
 *
 * @since 4.0.0
 *
 * @return array|bool False on failure, array of geo data on success.
 */
function va_listing_geo_coordinates() {
	global $post, $wpdb;

	$lat = get_post_meta( $post->ID, 'geo_lat', true );
	$lng = get_post_meta( $post->ID, 'geo_lng', true );

	// Using Vantage 4.0.0 or higher.
	if ( ! empty( $lat ) ) {
		return array( 'lat' => $lat, 'lng' => $lng );
	}

	// Support for pre 4.0.0 legacy db table if we get this far.
	// if ( $output = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM $wpdb->app_geodata WHERE post_id = %d", $post->ID ) ) ) {
	// 	return $output;
	// }

	$address = get_post_meta( $post->ID, 'address', true );

	// Last resort. Geo code on the fly based off the address.
	if ( empty( $address ) ) {
		return false;
	} else {
		// Geocode here and return coords.
		if ( class_exists( 'APP_Google_Geocoding' ) ) {
			APP_Google_Geocoding::insert_geo_data( $post->ID, $address );
		}
	}

	return false;
}

/**
 * Returns data attributes for each listing
 * which are primarily used by the map info bubble.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_listing_data_values() {
	global $post;

	$output = array();

	$geo_fields = APP_Google_Geocoding::get_post_meta( $post->ID );

	$data['id']        = $post->ID;
	$data['title']     = the_title_attribute( array( 'post' => $post, 'echo' => false ) );
	$data['permalink'] = get_permalink( $post->ID );
	$data['address']   = $geo_fields['raw_address'];
	$data['lat']       = $geo_fields['lat'];
	$data['lng']       = $geo_fields['lng'];

	$image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'thumbnail' );

	if ( is_array( $image ) && isset( $image[ 0 ] ) ) {
		$data['image'] = $image[ 0 ];
	} else {
		$data['image'] = va_placeholder_img_src();
	}

	// Build out the data attributes.
	foreach ( $data as $key => $value ) {
		$output[] .= sprintf( 'data-%s="%s"', $key, esc_attr( strip_tags( $value ) ) );
	}

	return implode( ' ', $output );
}
add_filter( 'va_listing_data_attributes', 'va_listing_data_values' );

/**
 * Returns base url for listings.
 *
 * @since 1.0.0
 *
 * @return string
 */
function va_listings_base_url() {
	global $va_options;

	$url = '';
	$base = trailingslashit( get_bloginfo( 'url' ) );

	if ( is_tax( VA_LISTING_CATEGORY ) || is_tax( VA_LISTING_TAG ) ) {
		$url = get_term_link( get_queried_object() );
	}

	if ( is_post_type_archive( VA_LISTING_PTYPE ) || is_front_page() ) {
		$url = $va_options->listing_permalink;
		$url = trailingslashit( $base . $url );
	}

	return $url;
}

if ( ! function_exists( 'va_editor' ) ) :
/**
 * Renders the WP textarea editor on the front end.
 *
 * @todo We don't use this so maybe deprecate? Listings module provides wp_editor().
 *
 * @see wp_editor()
 *
 * @since 1.4.0
 * @since 4.0.0 Added tinymce params and removed quicktags.
 *
 * @param string $content   Initial content for the editor.
 * @param string $editor_id HTML ID attribute value for the textarea and TinyMCE. Can only be /[a-z]+/.
 * @param array  $settings  See _WP_Editors::editor().
 */
function va_editor( $content = '', $editor_id = '', $settings = array() ) {

	/**
	 * Filter the wp_editor default settings.
	 *
	 * @since 1.0.0
	 */
	$settings = apply_filters( 'va_editor_settings', array(
		'media_buttons' => false,
		'textarea_name' => $editor_id,
		'textarea_rows' => 10,
		'quicktags'     => false,
		'tinymce'       => array(
			'plugins'                       => 'lists,paste,tabfocus,wplink,wordpress',
			'paste_as_text'                 => true,
			'paste_auto_cleanup_on_paste'   => true,
			'paste_remove_spans'            => true,
			'paste_remove_styles'           => true,
			'paste_remove_styles_if_webkit' => true,
			'paste_strip_class_attributes'  => true,
			'toolbar1'                      => 'bold,italic,|,bullist,numlist,|,link,unlink,|,undo,redo',
		),
	), $editor_id );

	wp_editor( $content, $editor_id, $settings );
}
endif;

if ( ! function_exists( 'va_the_archive_title' ) ) :
/**
 * Display the archive title based on the queried object.
 *
 * Based off the WordPress the_archive_title() function.
 * Added here so text strings can be translated with theme.
 *
 * @since 4.0.0
 *
 * @see the_archive_title()
 *
 * @param string $before Optional. Content to prepend to the title. Default empty.
 * @param string $after  Optional. Content to append to the title. Default empty.
 * @return string
 */
function va_the_archive_title( $before = '', $after = '' ) {

	$title = va_get_the_archive_title();

	if ( ! empty( $title ) ) {
		echo $before . $title . $after;
	}
}
endif;

if ( ! function_exists( 'va_get_the_archive_title' ) ) :
/**
 * Retrieve the archive title based on the queried object.
 * Based off the WordPress core function. Forked so we can
 * extract the text strings for easy translation.
 *
 * @since 4.0.0
 *
 * @return string Archive title.
 */
function va_get_the_archive_title() {

	if ( is_category() ) {
		$title = single_cat_title( '', false );
	} elseif ( is_tag() ) {
		$title = single_tag_title( '', false );
	} elseif ( is_author() ) {
		$title = sprintf( __( 'Author: %s', APP_TD ), '<span class="vcard">' . get_the_author() . '</span>' );
	} elseif ( is_year() ) {
		$title = sprintf( __( 'Year: %s', APP_TD ), get_the_date( _x( 'Y', 'yearly archives date format' ) ) );
	} elseif ( is_month() ) {
		$title = sprintf( __( 'Month: %s', APP_TD ), get_the_date( _x( 'F Y', 'monthly archives date format' ) ) );
	} elseif ( is_day() ) {
		$title = sprintf( __( 'Day: %s', APP_TD ), get_the_date( _x( 'F j, Y', 'daily archives date format' ) ) );
	} elseif ( is_tax( 'post_format' ) ) {
		if ( is_tax( 'post_format', 'post-format-aside' ) ) {
			$title = _x( 'Asides', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-gallery' ) ) {
			$title = _x( 'Galleries', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-image' ) ) {
			$title = _x( 'Images', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-video' ) ) {
			$title = _x( 'Videos', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-quote' ) ) {
			$title = _x( 'Quotes', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-link' ) ) {
			$title = _x( 'Links', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-status' ) ) {
			$title = _x( 'Statuses', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-audio' ) ) {
			$title = _x( 'Audio', 'post format archive title', APP_TD );
		} elseif ( is_tax( 'post_format', 'post-format-chat' ) ) {
			$title = _x( 'Chats', 'post format archive title', APP_TD );
		}
	} elseif ( is_post_type_archive() ) {
		$title = sprintf( __( 'Archives: %s', APP_TD ), post_type_archive_title( '', false ) );
	} elseif ( is_tax() ) {
		$tax = get_taxonomy( get_queried_object()->taxonomy );
		/* translators: 1: Taxonomy singular name, 2: Current taxonomy term */
		$title = sprintf( __( '%1$s: %2$s' ), $tax->labels->singular_name, single_term_title( '', false ) );
	} else {
		$title = __( 'Archives', APP_TD );
	}

	/**
	 * Filter the archive title.
	 *
	 * @since 4.0.0
	 *
	 * @param string $title Archive title to be displayed.
	 */
	return apply_filters( 'va_get_the_archive_title', $title );
}
endif;

/**
 * Inserts the search bar in the header.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_add_header_search_bar() {
	echo '<div class="top-bar-left">';
	get_template_part( 'searchform-listing' );
	echo '</div>';
}
add_action( 'va_header_top_bar_left', 'va_add_header_search_bar' );

/**
 * Inserts the title tag in the single listing item head.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_add_listing_item_title_tag() {

	the_title( sprintf( '<h4 class="entry-title"><a href="%s" rel="bookmark">', esc_url( get_permalink() ) ), '</a></h4>' );
}
add_action( 'va_listing_item_head', 'va_add_listing_item_title_tag' );

/**
 * Inserts the meta in the single listing item head.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_add_listing_item_meta() {
?>
	<li class="listing-address"><?php echo the_listing_address(); ?></li>
	<?php if ( $phone = get_post_meta( get_the_ID(), 'phone', true ) ) { ?>
		<li class="listing-phone fa-icon fa-phone"><a href="tel:<?php echo esc_attr( va_phone_url( $phone ) ); ?>" class="text-muted"><?php echo $phone; ?></a></li>
	<?php } ?>
	<li class="listing-categories">
		<?php
		if ( $terms = get_the_terms( get_the_ID(), VA_LISTING_CATEGORY ) ) {
			foreach ( $terms as $term ) {
				//if ( 0 == $term->parent ) {
					echo '<span class="listing-parent-category label"><a href="' . get_term_link( $term->slug, VA_LISTING_CATEGORY ) . '">' . $term->name . '</a></span>';
				//}
				//unset( $term );
			}
		}
		?>
	</li>
<?php
}
add_action( 'va_listing_item_meta', 'va_add_listing_item_meta' );

/**
 * Inserts the title and categories in the single listing page head.
 *
 * @since 4.0.0
 *
 * @return string
 */
function va_add_listing_single_head() {

	the_title( '<h1 class="entry-title">', '</h1>' );
?>

	<div class="entry-categories">
		<?php the_listing_categories(); ?>
	</div><!-- .entry-categories -->
<?php
}
add_action( 'va_listing_single_head', 'va_add_listing_single_head' );

/**
 * Hides the custom post template for pages on WordPress 4.6 and older.
 *
 * @link https://make.wordpress.org/core/2016/11/03/post-type-templates-in-4-7/
 *
 * @since 4.0.0
 *
 * @param array $post_templates Array of page templates. Keys are filenames, values are translated names.
 * @return array Filtered array of page templates.
 */
function va_exclude_page_templates( $post_templates ) {

	if ( version_compare( $GLOBALS['wp_version'], '4.7', '<' ) ) {
		unset( $post_templates['page-templates/template-post-full-width.php'] );
	}

	return $post_templates;
}
add_filter( 'theme_page_templates', 'va_exclude_page_templates' );

function va_google_geo_fields( $fields ) {
	$fields['raw_address'] = 'address';
	return $fields;
}
add_filter( 'appthemes_google_geo_fields', 'va_google_geo_fields' );

/**
 * Internal compat function to mimic mb_strimwidth().
 *
 * @since 4.1.0
 *
 * (PHP 4 &gt;= 4.0.6, PHP 5, PHP 7)<br/>
 * Get truncated string with specified width
 * @link http://php.net/manual/en/function.mb-strimwidth.php
 *
 * @param string $str        The string being decoded.
 * @param int    $start      The start position offset. Number of characters
 *                           from the beginning of string. First character is 0.
 * @param int    $width      The width of the desired trim.
 * @param string $trimmarker [optional] A string that is added to the end of
 *                           string when string is truncated.
 *
 * @return string The truncated string. If <i>trimmarker</i> is set, <i>trimmarker</i> is appended to the return value.
 */
function va_strimwidth( $str, $start, $width, $trimmarker = '' ) {
	if ( function_exists( 'mb_strimwidth' ) ) {
		$str = mb_strimwidth( $str, $start, $width, $trimmarker );
	} else {
		$trimmarker = ( mb_strlen( $str ) - $start <= $width ) ? '' : $trimmarker;
		$str = rtrim( mb_substr( $str, $start, max( 0, $width - mb_strlen( $trimmarker ) ) ) ) . $trimmarker;
	}

	return $str;
}