<?php
/**
 * Sets up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support for post thumbnails.
 *
 * @package Vantage
 * @since 4.0.0
 */

if ( ! function_exists( 'va_setup_theme_support' ) ) :
/**
 * Runs the after setup theme action hook.
 *
 * @since 4.0.0
 */
function va_setup_theme_support() {
	global $va_options, $vantage;

	/**
	 * Let WordPress manage the document title.
	 *
	 * @since 4.0.0
	 */
	add_theme_support( 'title-tag' );

	/**
	 * Enable support for post thumbnails on posts and pages.
	 *
	 * @since 1.0.0
	 */
	add_theme_support( 'post-thumbnails' );

	/**
	 * Set default post thumbnail size.
	 *
	 * Use a max width of 1200 and unlimited height for better responsive support.
	 *
	 * @since 4.0.0
	 */
	set_post_thumbnail_size( 1200, 9999 );

	/**
	 * Add default posts and comments RSS feed links to head.
	 *
	 * @since 1.0.0
	 */
	add_theme_support( 'automatic-feed-links' );

	/**
	 * Enable support for a custom background image.
	 *
	 * @since 4.0.0
	 */
	add_theme_support( 'custom-background', apply_filters( 'va_custom_background_args', array(
		'default-color'    => '#F3F3F4',
		'default-image'    => '',
		'wp-head-callback' => 'va_custom_background_cb',
	) ) );

	/**
	 * Enable support for a custom logo.
	 *
	 * @since 4.0.0
	 */
	add_theme_support( 'custom-logo', array(
		'height'      => 50,
		'width'       => 150,
		'flex-width'  => true,
		'flex-height' => false,
	) );

	/**
	 * Switch default core markup to output valid HTML5.
	 *
	 * @since 4.0.0
	 */
	add_theme_support( 'html5', array(
		'comment-list',
		'comment-form',
		'search-form',
	) );

	/**
	 * Indicate widget sidebars can use selective refresh in the Customizer.
	 *
	 * @since 4.0.0
	 */
	add_theme_support( 'customize-selective-refresh-widgets' );

	/**
	 * Custom WordPress editor stylesheet.
	 *
	 * @since 4.0.0
	 */
	 // Minimize prod or show expanded in dev.
	$min = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
	add_editor_style( get_template_directory_uri() . "/assets/css/editor-style{$min}.css" );

	/**
	 * We want more control over the language file location.
	 *
	 * @todo Remove this once the function has been deprecated from theme-framework.
	 *
	 * @since 4.0.0
	 */
	remove_action( 'appthemes_theme_framework_loaded', 'appthemes_load_textdomain' );

	/**
	 * Add support for language files.
	 *
	 * Looks in WP_LANG_DIR . '/themes/' first otherwise
	 * For example: wp-content/languages/themes/vantage-de_DE.mo
	 *
	 * Otherwise defaults to: "wp-content/themes/vantage/languages/$domain . '-' . $locale.mo"
	 *
	 * @since 4.0.0
	 */
	load_theme_textdomain( APP_TD, get_template_directory() . '/languages' );

	/**
	 * Disable the default WordPress gallery styles so we can use our own.
	 *
	 * @since 4.0.0
	 */
	add_filter( 'use_default_gallery_style', '__return_false' );

	/**
	 * AppThemes specific theme support items.
	 *
	 * @todo Add comment blocks.
	 */

	// add_theme_support( 'app-search-index' );

	/**
	 * Page to redirect to after theme install/update.
	 *
	 * @since 1.0.0
	 */
	add_theme_support( 'app-versions', array(
		'update_page'     => 'admin.php?page=app-setup&firstrun=1',
		'current_version' => VA_VERSION,
		'option_key'      => 'vantage_version',
	) );

	add_theme_support( 'app-wrapping' );

	add_theme_support( 'app-login', array(
		'login'    => 'form-login.php',
		'register' => 'form-registration.php',
		'recover'  => 'form-password-recovery.php',
		'reset'    => 'form-password-reset.php',
	) );

	add_theme_support( 'app-payments', array(
		'options' => $va_options,
	) );

	add_theme_support( 'app-price-format', array(
		'currency_default'    => $va_options->currency_code,
		'currency_identifier' => $va_options->currency_identifier,
		'currency_position'   => $va_options->currency_position,
		'thousands_separator' => $va_options->thousands_separator,
		'decimal_separator'   => $va_options->decimal_separator,
		'hide_decimals'       => (bool) ( ! $va_options->decimal_separator ),
	) );

	// add_theme_support( 'app-geo-2', array(
	// 	'options' => $va_options,
	// ) );

	add_theme_support( 'app-term-counts', array(
		'post_type'   => array( VA_LISTING_PTYPE ),
		'post_status' => array( 'publish' ),
		'taxonomy'    => array( VA_LISTING_CATEGORY ),
	) );

	add_theme_support( 'app-feed', array(
		'post_type'          => VA_LISTING_PTYPE,
		'blog_template'      => 'index.php',
		'alternate_feed_url' => '',
	) );

	add_theme_support( 'app-html-term-description', array(
			'taxonomy' => array(
				VA_LISTING_CATEGORY,
				VA_LISTING_TAG,
				'category',
				'post_tag',
			),
		)
	);

	add_theme_support( 'app-form-builder', array(
		'show_in_menu'       => true,
		'register_post_type' => in_array( 'custom_form', $vantage->{VA_LISTING_PTYPE}->options->features ),
	) );

	add_theme_support( 'app-addons-mp', array(
		'product' => array( 476 ),
	) );

	add_theme_support( 'app-require-updater', true );
	add_theme_support( 'app-media-manager' );
}
endif;
add_action( 'after_setup_theme', 'va_setup_theme_support' );

/**
 * Sets the content width in pixels, based on the theme's design and stylesheet.
 *
 * Priority 0 to make it available to lower priority callbacks.
 *
 * @global int $content_width
 *
 * @since 4.0.0
 */
function va_content_width() {
	/**
	 * Filter the content width in pixels.
	 *
	 * @since 4.0.0
	 *
	 * @param int $width The content width in pixels.
	 */
	$GLOBALS['content_width'] = apply_filters( 'va_content_width', 745 );
}
add_action( 'after_setup_theme', 'va_content_width', 0 );
