<?php
/**
 * User Profile views
 *
 * Controller for the user profile. Handles data saves, etc.
 *
 * @package Vantage\Views
 * @author AppThemes
 * @since 4.0.0
 */

/**
 * User Profile view page
 *
 * @since 4.0.0
 */
class VA_Edit_User_Profile extends APP_User_Profile {

	function __construct() {
		parent::__construct();

		add_action( 'personal_options_update', array( $this, 'personal_options_update' ) );
		add_action( 'wp_ajax_vantage_delete_cover_image',        array( $this, 'delete_cover_image' ) );
		add_action( 'wp_ajax_nopriv_vantage_delete_cover_image', array( $this, 'delete_cover_image' ) );
	}

	function update() {
		global $user_ID;

		if ( ! isset( $_POST['action'] ) || 'app-edit-profile' != $_POST['action'] ) {
			return;
		}

		if ( absint( $_POST['user_id'] ) !== absint( $user_ID ) ) {
			return appthemes_add_notice( 'incorrect_user_id', __( '<strong>ERROR</strong>: Incorrect user ID.', APP_TD ), 'error' );
		}

		parent::update();
	}

	/**
	 * Updates custom profile fields.
	 *
	 * @param type $user_ID
	 * @return type
	 */
	public function personal_options_update( $user_ID ) {
		if ( ! isset( $_POST['action'] ) || 'app-edit-profile' != $_POST['action'] ) {
			return;
		}

		if ( is_uploaded_file( $_FILES[ 'cover_image' ]['tmp_name'] ) ) {
			$this->add_cover_image( $_FILES, 'cover_image' );
		}
	}

	/**
	 * Process the cover image upload.
	 *
	 * @since   4.0.0
	 *
	 * @param	array $files The $_FILES array object.
	 *
	 * @return false|int $attachment_id The attachment id or false if upload failed.
	 */
	protected function add_cover_image( $files, $field ) {
		global $user_ID;

		$upload = $_FILES[ $field ];

		// We don't have an image so bail.
		if ( $upload['size'] == 0 ) {
			wp_die( __( 'The uploaded file has a size of zero. Please try again.', APP_TD ) );
		}

		// Limit file upload sizes.
		$max_size = va_upload_size_limit();

		if ( $upload['size'] >= $max_size ) {
			wp_die( __( 'The uploaded file is too large. Please try again.', APP_TD ) );
		}

		// Check for valid file type and extension.
		$wp_filetype = wp_check_filetype_and_ext( $upload['tmp_name'], $upload['name'] );

		// Check for valid mime type.
		if ( ! wp_match_mime_types( 'image', $wp_filetype['type'] ) ) {
			wp_die( __( 'The uploaded file is not a valid image. Please try again.', APP_TD ) );
		}

		// We made it past security so let's handle the upload.

		// WordPress API for image uploads.
		include_once( ABSPATH . 'wp-admin/includes/image.php' );
		include_once( ABSPATH . 'wp-admin/includes/file.php' );
		include_once( ABSPATH . 'wp-admin/includes/media.php' );

		// Upload the image.
		$file = wp_handle_upload( $upload, array( 'test_form' => false ) );

		if ( isset( $file['error'] ) ) {
			wp_die( $file['error'],  __( 'Image Upload Error', APP_TD ) );
		}

		// Delete any previous cover image, if any.
		if ( $old_id = get_user_meta( $user_ID, $field, true ) ) {
			wp_delete_attachment( $old_id );
			delete_user_meta( $user_ID, $field );
		}

		$name       = $upload['name'];
		$name_parts = pathinfo( $name );
		$name       = trim( substr( $name, 0, -( 1 + strlen( $name_parts['extension'] ) ) ) );

		// Construct the object array.
		$object = array(
			'post_mime_type' => $file['type'],
			'guid'           => $file['url'],
			'post_title'     => sanitize_title( $name ),
			'post_content'   => '',
			'post_excerpt'   => '',
			'context'        => 'author-cover',
		);

		// Save the image in the WordPress db.
		$attachment_id = wp_insert_attachment( $object, $file['file'] );

		// Add the metadata (e.g. width, height, etc).
		if ( ! is_wp_error( $attachment_id ) ) {
			wp_update_attachment_metadata( $attachment_id, wp_generate_attachment_metadata( $attachment_id, $file['file'] ) );
		}

		// Link the uploaded image to the user.
		update_user_meta( $user_ID, $field, $attachment_id );

		return $attachment_id;
	}

	/**
	 * Process the ajax delete cover image action.
	 *
	 * @since   4.0.0
	 *
	 * @return  array The ajax response.
	 */
	function delete_cover_image() {
		global $user_ID;

		// Make sure it's a legit request.
		check_ajax_referer( 'vantage_delete_cover_image_' . $user_ID, 'nonce' );

		// We need certain params to proceed.
		if ( ! isset( $user_ID ) ) {
			wp_send_json_error( __( 'User ID is required.', APP_TD ) );
		}

		// Delete the cover image.
		if ( $old_id = get_user_meta( $user_ID, 'cover_image', true ) ) {
			$post = wp_delete_attachment( $old_id );
			delete_user_meta( $user_ID, 'cover_image' );
		}

		// Image was not found.
		if ( ! $post ) {
			wp_send_json_error( __( 'Image not found.', APP_TD ) );
		}

		// Return the response.
		wp_send_json_success();
	}

	function enqueue_scripts() {
		wp_enqueue_script( 'user-profile' );
	}
}
