<?php
/**
 * Listings search results page classes.
 *
 * @package Vantage\Views
 * @author AppThemes
 * @since 4.0.0
 */

/**
 * Logic for the listings search results page.
 *
 * @since 4.0.0
 */
class VA_Listing_Search_View extends APP_View {

	/**
	 * Init and whitelist search parameters.
	 *
	 * @since  1.0.0
	 *
	 * @return none
	 */
 	function init() {
 		global $wp;

		// Use a different keyword search param since 's' is reserved for the blog.
 		$wp->add_query_var( 'ls' );

		// Declare a 'search type' param (defaults to listing) so things like events can be added later.
 		$wp->add_query_var( 'st' );

		$wp->add_query_var( 'location' );
		$wp->add_query_var( 'radius' );
		$wp->add_query_var( 'lat' );
		$wp->add_query_var( 'lng' );
 	}

	/**
	* Determine what page(s) this class should load on.
	*
	* @since  1.0.0
	*
	* @return bool true if successful, false otherwise
	*/
 	function condition() {
 		return ( isset( $_GET['ls'] ) || get_query_var( 'location' ) ) && ( ( isset( $_GET['st'] ) && $_GET['st'] == 'listing' ) || ! isset( $_GET['st'] ) );
 	}

	/**
	 * Called after the query variable object is created, but before the actual query is run.
	 *
	 * @since 1.0.0
	 *
	 * @uses APP_Google_Maps
	 *
	 * @param WP_Query $wp_query The WP_Query instance (passed by reference).
	 */
	function parse_query( $wp_query ) {
		global $va_options;

		// Override default parameters.
		$wp_query->set( 'ls', trim( get_query_var( 'ls' ) ) );
		$wp_query->set( 's', get_query_var( 'ls' ) );
		$wp_query->set( 'post_type', VA_LISTING_PTYPE );
		$wp_query->set( 'posts_per_page', $va_options->listings_per_page );

		// Filter by listing cat if needed.
		if ( isset( $_GET['listing_cat'] ) ) {
			$tax_query[] = array(
				'taxonomy' => VA_LISTING_CATEGORY,
				'terms'    => $_GET['listing_cat'],
			);
			$wp_query->set( 'tax_query', $tax_query );
		}

		// Get post ids of any radius geo search results
		// so we can filter the next query based off of it.
		$post_ids = APP_Google_Maps::posts_clauses();

		// If the array isn't empty, set the post ids to filter from.
		if ( $post_ids ) {
			$wp_query->set( 'post__in', array_keys( array_filter( $post_ids ) ) );
		}

		if ( 'title' === $wp_query->get( 'orderby' ) ) {
			$wp_query->set( 'order', 'ASC' );
		}
	}

	/**
	 * Filters the search SQL that is used in the WHERE clause of WP_Query.
	 *
	 * @since 1.x.x
	 * @since 4.0.0 Rebuilt for better speed, security, and dropped older WP support.
	 *              Removed va_search_index_enabled() support.
	 *
	 * @param string   $where    Search SQL for WHERE clause.
	 * @param WP_Query $wp_query The current WP_Query object.
	 * @return string  $search
	 */
	function posts_search( $where, $wp_query ) {
		global $wpdb;

		$q = $wp_query->query_vars;

		if ( empty( $q['search_terms'] ) ) {
			return $where;
		}

		// Start code fork from WP_Query::parse_search().
		$search    = '';
		$searchand = '';
		$n         = ! empty( $q['exact'] ) ? '' : '%';

		foreach ( $q['search_terms'] as $term ) {
			$like = $n . $wpdb->esc_like( $term ) . $n;
			$search .= $wpdb->prepare( "{$searchand}(($wpdb->posts.post_title LIKE %s) OR ($wpdb->posts.post_content LIKE %s) OR (tter.name LIKE %s))", $like, $like, $like );
			$searchand = ' OR ';
		}

		if ( ! empty( $search ) ) {
			$search = " AND ({$search}) ";
			if ( ! is_user_logged_in() )
				$search .= " AND ($wpdb->posts.post_password = '') ";
		}
		// End fork.

		return $search;
	}

	/**
	 * Filters all query clauses at once, for convenience.
	 *
	 * Covers the WHERE, GROUP BY, JOIN, ORDER BY, DISTINCT,
	 * fields (SELECT), and LIMITS clauses.
	 * Enables searching of ALL taxonomies by name.
	 * Using custom prefixed aliases since they may collide with other plugins.
	 *
	 * @since 1.x.x
	 * @since 4.0.0 Added support for all custom taxonomies.
	 *              Removed va_search_index_enabled() support.
	 *
	 * @param array    $clauses  The list of clauses for the query.
 	 * @param WP_Query $wp_query The WP_Query instance.
	 */
	function posts_clauses( $clauses, $wp_query ) {
		global $wpdb;

		$clauses['join'] .= "
			LEFT JOIN $wpdb->term_relationships trel ON ($wpdb->posts.ID = trel.object_id)
			INNER JOIN $wpdb->term_taxonomy ttax ON (ttax.term_taxonomy_id = trel.term_taxonomy_id)
			INNER JOIN $wpdb->terms tter ON (ttax.term_id = tter.term_id)
		";

		$clauses['distinct'] = 'DISTINCT';

		return $clauses;
	}
}
